"""
Settings for external services to query for autocomplete
when assigning skos Matches
"""

search_types = (
    ('KeywordSearch?', 'Keyword Search'),
    ('PrefixSearch?', 'Prefix Search')
)


class DbpediaAC(object):
    """
    Dbpedia
    """
    endpoint = 'http://lookup.dbpedia.org/api/search/'
    search_type = 'PrefixSearch?'

    def payload(self, q):
        """
        returns a dictionary containing arguments to be
        passed in the URL’s query string
        """
        return {'QueryString': q}

    def get_url(self):
        """
        service's URL
        """
        return self.endpoint + self.search_type

    def parse_response(self, response):
        """
        parses JSON response to return a list containing
        data in format 'uri - label'
        """
        return [str(x['uri']) + ' - ' + str(x['label']) for x in response['results']]


class GndAC(object):
    """
    GND
    """
    endpoint = 'https://lobid.org/gnd/search?'

    def payload(self, q):
        return {'format': 'json:preferredName', 'q': q}

    def get_url(self):
        return self.endpoint

    def parse_response(self, response):
        return [str(x['id']) + ' - ' + str(x['label']) for x in response]


class GemetAC(object):
    """
    GEMET Thesaurus
    """
    endpoint = 'https://www.eionet.europa.eu/gemet/'
    search_type = 'getConceptsMatchingKeyword?'

    def payload(self, q):
        return {'search_mode': '4', 'keyword': q}

    def get_url(self):
        return self.endpoint + self.search_type

    def parse_response(self, response):
        return [str(x['uri']) + ' - ' + str(x['preferredLabel']['string']) for x in response]


class FishAC(object):
    """
    FISH Vocabularies
    """
    endpoint = 'https://www.heritagedata.org/live/services/'
    search_type = 'getConceptLabelMatch?'
    scheme_dict = {
        "FISH Event Types Thesaurus": "http://purl.org/heritagedata/schemes/agl_et",
        "FISH Archaeological Sciences Thesaurus": "http://purl.org/heritagedata/schemes/560",
        "FISH Thesaurus of Monument Types": "http://purl.org/heritagedata/schemes/eh_tmt2",
        "FISH Archaeological Objects Thesaurus": "http://purl.org/heritagedata/schemes/mda_obj"
    }

    def payload(self, scheme, q):
        payload = {'schemeURI': scheme, 'contains': q}
        return payload

    def get_url(self):
        return self.endpoint + self.search_type

    def parse_response(self, response):
        return [str(x['uri']) + ' - ' + str(x['label']) for x in response]


ENDPOINT = {
    'Dbpedia': DbpediaAC(),
    'GND': GndAC(),
    'GEMET': GemetAC(),
    'FISH Event Types Thesaurus': FishAC(),
    'FISH Archaeological Sciences Thesaurus': FishAC(),
    'FISH Thesaurus of Monument Types': FishAC(),
    'FISH Archaeological Objects Thesaurus': FishAC()
}

ENDPOINT_CHOICES = [(key, key) for key, value in ENDPOINT.items()]



from crispy_forms.layout import LayoutObject, TEMPLATE_PACK
from django.shortcuts import render
from django.template.loader import render_to_string


class Formset(LayoutObject):
    """
    Layout object. It renders an entire formset, as though it were a Field.

    Example:

    Formset("attached_files_formset")
    """

    template = "vocabs/formset.html" % TEMPLATE_PACK

    def __init__(self, formset_name_in_context, template=None):
        self.formset_name_in_context = formset_name_in_context
        
        # crispy_forms/layout.py:302 requires us to have a fields property
        self.fields = []

        # Overrides class variable with an instance level variable
        if template:
            self.template = template

    def render(self, form, form_style, context, template_pack=TEMPLATE_PACK):
        formset = context[self.formset_name_in_context]
        return render_to_string(self.template, {'formset': formset})


from django.contrib import admin
from .models import *
from reversion.admin import VersionAdmin
from mptt.admin import MPTTModelAdmin


# With object permissions support
@admin.register(SkosConcept)
class SkosConceptAdmin(MPTTModelAdmin, VersionAdmin):
	pass


class SkosCollectionAdmin(VersionAdmin):
	pass


class SkosConceptSchemeAdmin(VersionAdmin):
	pass


admin.site.register(SkosCollection, SkosCollectionAdmin)
admin.site.register(SkosConceptScheme, SkosConceptSchemeAdmin)
admin.site.register(ConceptSchemeTitle)
admin.site.register(ConceptSchemeDescription)
admin.site.register(ConceptSchemeSource)
admin.site.register(CollectionLabel)
admin.site.register(CollectionNote)
admin.site.register(CollectionSource)
admin.site.register(ConceptLabel)
admin.site.register(ConceptNote)
admin.site.register(ConceptSource)



from django.views.generic.detail import DetailView
from django.views.generic.edit import DeleteView
from django.utils.decorators import method_decorator
from django.urls import reverse_lazy
from django_tables2 import RequestConfig
from .models import SkosConcept, SkosConceptScheme, SkosCollection
from .forms import *
from .tables import *
from .filters import SkosConceptListFilter, SkosConceptSchemeListFilter, SkosCollectionListFilter
from browsing.browsing_utils import GenericListView, BaseCreateView, BaseUpdateView
from .rdf_utils import *
from django.shortcuts import render
from django.http import HttpResponse
import time
import datetime
from django.contrib.auth.decorators import login_required, permission_required
from reversion.models import Version
from django.db import transaction
from django.shortcuts import redirect
from .skos_import import *
from django.contrib import messages


class BaseDetailView(DetailView):

    def get_context_data(self, **kwargs):
        context = super(BaseDetailView, self).get_context_data(**kwargs)
        context['history'] = Version.objects.get_for_object(self.object)
        return context


######################################################################
#
# SkosConceptScheme
#
######################################################################

class SkosConceptSchemeListView(GenericListView):
    model = SkosConceptScheme
    table_class = SkosConceptSchemeTable
    filter_class = SkosConceptSchemeListFilter
    formhelper_class = SkosConceptSchemeFormHelper
    init_columns = [
        'id',
        'title',
    ]

    def get_all_cols(self):
        all_cols = list(self.table_class.base_columns.keys())
        return all_cols

    def get_context_data(self, **kwargs):
        context = super(SkosConceptSchemeListView, self).get_context_data()
        context[self.context_filter_name] = self.filter
        togglable_colums = [x for x in self.get_all_cols() if x not in self.init_columns]
        context['togglable_colums'] = togglable_colums
        return context

    def get_table(self, **kwargs):
        table = super(GenericListView, self).get_table()
        RequestConfig(self.request, paginate={
            'page': 1, 'per_page': self.paginate_by
        }).configure(table)
        default_cols = self.init_columns
        all_cols = self.get_all_cols()
        selected_cols = self.request.GET.getlist("columns") + default_cols
        exclude_vals = [x for x in all_cols if x not in selected_cols]
        table.exclude = exclude_vals
        return table


class SkosConceptSchemeDetailView(BaseDetailView):
    model = SkosConceptScheme
    template_name = 'vocabs/skosconceptscheme_detail.html'

    def get_context_data(self, **kwargs):
        context = super(SkosConceptSchemeDetailView, self).get_context_data(**kwargs)
        context["concepts"] = SkosConcept.objects.filter(scheme=self.kwargs.get('pk'))
        return context


class SkosConceptSchemeCreate(BaseCreateView):
    model = SkosConceptScheme
    form_class = SkosConceptSchemeForm
    success_url = None

    def get_context_data(self, **kwargs):
        data = super(SkosConceptSchemeCreate, self).get_context_data(**kwargs)
        if self.request.POST:
            data['titles'] = ConceptSchemeTitleFormSet(self.request.POST)
            data['descriptions'] = ConceptSchemeDescriptionFormSet(self.request.POST)
            data['sources'] = ConceptSchemeSourceFormSet(self.request.POST)
        else:
            data['titles'] = ConceptSchemeTitleFormSet()
            data['descriptions'] = ConceptSchemeDescriptionFormSet()
            data['sources'] = ConceptSchemeSourceFormSet()
        return data

    def form_valid(self, form):
        context = self.get_context_data()
        titles = context['titles']
        descriptions = context['descriptions']
        sources = context['sources']
        with transaction.atomic():
            form.instance.created_by = self.request.user
            # cs should be saved first because fk object are related to it
            self.object = form.save(commit=False)
            if titles.is_valid():
                titles.instance = self.object
                titles.save(commit=False)
            else:
                return super(SkosConceptSchemeCreate, self).form_invalid(form)
            if descriptions.is_valid():
                descriptions.instance = self.object
                descriptions.save(commit=False)
            else:
                return super(SkosConceptSchemeCreate, self).form_invalid(form)
            if sources.is_valid():
                sources.instance = self.object
                sources.save(commit=False)
            else:
                return super(SkosConceptSchemeCreate, self).form_invalid(form)
            self.object = form.save()
            titles.save()
            descriptions.save()
            sources.save()

        return super(SkosConceptSchemeCreate, self).form_valid(form)

    def get_success_url(self):
        return reverse_lazy('vocabs:skosconceptscheme_detail', kwargs={'pk': self.object.pk})

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(SkosConceptSchemeCreate, self).dispatch(*args, **kwargs)


class SkosConceptSchemeUpdate(BaseUpdateView):
    model = SkosConceptScheme
    form_class = SkosConceptSchemeForm
    permission_required = (
        'view_skosconceptscheme',
        'change_skosconceptscheme',
        'delete_skosconceptscheme',
    )
    success_url = None

    def get_context_data(self, **kwargs):
        data = super(SkosConceptSchemeUpdate, self).get_context_data(**kwargs)
        if self.request.POST:
            data['titles'] = ConceptSchemeTitleFormSet(
                self.request.POST, instance=self.object
            )
            data['descriptions'] = ConceptSchemeDescriptionFormSet(
                self.request.POST, instance=self.object
            )
            data['sources'] = ConceptSchemeSourceFormSet(
                self.request.POST, instance=self.object
            )
        else:
            data['titles'] = ConceptSchemeTitleFormSet(instance=self.object)
            data['descriptions'] = ConceptSchemeDescriptionFormSet(instance=self.object)
            data['sources'] = ConceptSchemeSourceFormSet(instance=self.object)
        return data

    def form_valid(self, form):
        context = self.get_context_data()
        titles = context['titles']
        descriptions = context['descriptions']
        sources = context['sources']
        with transaction.atomic():
            if titles.is_valid():
                titles.instance = self.object
                titles.save()
            else:
                # raise forms.ValidationError("Both fields should be filled")
                return super(SkosConceptSchemeUpdate, self).form_invalid(form)
            if descriptions.is_valid():
                descriptions.instance = self.object
                descriptions.save()
            else:
                return super(SkosConceptSchemeUpdate, self).form_invalid(form)
            if sources.is_valid():
                sources.instance = self.object
                sources.save()
            else:
                return super(SkosConceptSchemeUpdate, self).form_invalid(form)
        return super(SkosConceptSchemeUpdate, self).form_valid(form)

    def get_success_url(self):
        return reverse_lazy('vocabs:skosconceptscheme_detail', kwargs={'pk': self.object.pk})

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(SkosConceptSchemeUpdate, self).dispatch(*args, **kwargs)


class SkosConceptSchemeDelete(DeleteView):
    model = SkosConceptScheme
    template_name = 'webpage/confirm_delete.html'
    success_url = reverse_lazy('vocabs:browse_schemes')

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(SkosConceptSchemeDelete, self).dispatch(*args, **kwargs)


######################################################################
#
# SkosCollection
#
######################################################################

class SkosCollectionListView(GenericListView):
    model = SkosCollection
    table_class = SkosCollectionTable
    filter_class = SkosCollectionListFilter
    formhelper_class = SkosCollectionFormHelper
    init_columns = [
        'id',
        'name',
        'scheme',
    ]

    def get_all_cols(self):
        all_cols = list(self.table_class.base_columns.keys())
        return all_cols

    def get_context_data(self, **kwargs):
        context = super(SkosCollectionListView, self).get_context_data()
        context[self.context_filter_name] = self.filter
        togglable_colums = [x for x in self.get_all_cols() if x not in self.init_columns]
        context['togglable_colums'] = togglable_colums
        return context

    def get_table(self, **kwargs):
        table = super(GenericListView, self).get_table()
        RequestConfig(self.request, paginate={
            'page': 1, 'per_page': self.paginate_by
        }).configure(table)
        default_cols = self.init_columns
        all_cols = self.get_all_cols()
        selected_cols = self.request.GET.getlist("columns") + default_cols
        exclude_vals = [x for x in all_cols if x not in selected_cols]
        table.exclude = exclude_vals
        return table


class SkosCollectionDetailView(BaseDetailView):
    model = SkosCollection
    template_name = 'vocabs/skoscollection_detail.html'


class SkosCollectionCreate(BaseCreateView):
    model = SkosCollection
    form_class = SkosCollectionForm
    success_url = None

    def get_context_data(self, **kwargs):
        data = super(SkosCollectionCreate, self).get_context_data(**kwargs)
        if self.request.POST:
            data['labels'] = CollectionLabelFormSet(self.request.POST)
            data['notes'] = CollectionNoteFormSet(self.request.POST)
            data['sources'] = CollectionSourceFormSet(self.request.POST)
        else:
            data['labels'] = CollectionLabelFormSet()
            data['notes'] = CollectionNoteFormSet()
            data['sources'] = CollectionSourceFormSet()
        return data

    def form_valid(self, form):
        context = self.get_context_data()
        labels = context['labels']
        notes = context['notes']
        sources = context['sources']
        with transaction.atomic():
            form.instance.created_by = self.request.user
            self.object = form.save(commit=False)
            if labels.is_valid():
                labels.instance = self.object
                labels.save(commit=False)
            else:
                return super(SkosCollectionCreate, self).form_invalid(form)
            if notes.is_valid():
                notes.instance = self.object
                notes.save(commit=False)
            else:
                return super(SkosCollectionCreate, self).form_invalid(form)
            if sources.is_valid():
                sources.instance = self.object
                sources.save(commit=False)
            else:
                return super(SkosCollectionCreate, self).form_invalid(form)
            self.object = form.save()
            labels.save()
            notes.save()
            sources.save()
        return super(SkosCollectionCreate, self).form_valid(form)

    def get_initial(self):
        initial = super(SkosCollectionCreate, self).get_initial()
        if self.request.GET.get('scheme'):
            initial['scheme'] = SkosConceptScheme.objects.get(pk=self.request.GET.get('scheme'))
        return initial

    def get_success_url(self):
        return reverse_lazy('vocabs:skoscollection_detail', kwargs={'pk': self.object.pk})

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(SkosCollectionCreate, self).dispatch(*args, **kwargs)


class SkosCollectionUpdate(BaseUpdateView):
    model = SkosCollection
    form_class = SkosCollectionForm
    permission_required = (
        'view_skoscollection',
        'change_skoscollection',
        'delete_skoscollection',
    )
    success_url = None

    def get_context_data(self, **kwargs):
        data = super(SkosCollectionUpdate, self).get_context_data(**kwargs)
        if self.request.POST:
            data['labels'] = CollectionLabelFormSet(self.request.POST, instance=self.object)
            data['notes'] = CollectionNoteFormSet(self.request.POST, instance=self.object)
            data['sources'] = CollectionSourceFormSet(self.request.POST, instance=self.object)
        else:
            data['labels'] = CollectionLabelFormSet(instance=self.object)
            data['notes'] = CollectionNoteFormSet(instance=self.object)
            data['sources'] = CollectionSourceFormSet(instance=self.object)
        return data

    def form_valid(self, form):
        context = self.get_context_data()
        labels = context['labels']
        notes = context['notes']
        sources = context['sources']
        with transaction.atomic():
            if labels.is_valid():
                labels.instance = self.object
                labels.save()
            else:
                return super(SkosCollectionUpdate, self).form_invalid(form)
            if notes.is_valid():
                notes.instance = self.object
                notes.save()
            else:
                return super(SkosCollectionUpdate, self).form_invalid(form)
            if sources.is_valid():
                sources.instance = self.object
                sources.save()
            else:
                return super(SkosCollectionUpdate, self).form_invalid(form)
        return super(SkosCollectionUpdate, self).form_valid(form)

    def get_success_url(self):
        return reverse_lazy('vocabs:skoscollection_detail', kwargs={'pk': self.object.pk})

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(SkosCollectionUpdate, self).dispatch(*args, **kwargs)


class SkosCollectionDelete(DeleteView):
    model = SkosCollection
    template_name = 'webpage/confirm_delete.html'
    success_url = reverse_lazy('vocabs:browse_skoscollections')

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(SkosCollectionDelete, self).dispatch(*args, **kwargs)


######################################################################
#
# SkosConcept
#
######################################################################

class SkosConceptListView(GenericListView):
    model = SkosConcept
    table_class = SkosConceptTable
    filter_class = SkosConceptListFilter
    formhelper_class = SkosConceptFormHelper
    init_columns = [
        'id',
        'pref_label',
        'scheme',
    ]

    def get_queryset(self, **kwargs):
        qs = super(SkosConceptListView, self).get_queryset()
        return qs.order_by('id')


class SkosConceptDetailView(BaseDetailView):
    model = SkosConcept
    template_name = 'vocabs/skosconcept_detail.html'
    success_url = None

    def get_context_data(self, **kwargs):
        context = super(SkosConceptDetailView, self).get_context_data(**kwargs)
        return context


class SkosConceptCreate(BaseCreateView):
    model = SkosConcept
    form_class = SkosConceptForm

    def get_context_data(self, **kwargs):
        data = super(SkosConceptCreate, self).get_context_data(**kwargs)
        if self.request.POST:
            data['labels'] = ConceptLabelFormSet(self.request.POST)
            data['notes'] = ConceptNoteFormSet(self.request.POST)
            data['sources'] = ConceptSourceFormSet(self.request.POST)
        else:
            data['labels'] = ConceptLabelFormSet()
            data['notes'] = ConceptNoteFormSet()
            data['sources'] = ConceptSourceFormSet()
        return data

    def form_valid(self, form):
        context = self.get_context_data()
        labels = context['labels']
        notes = context['notes']
        sources = context['sources']
        with transaction.atomic():
            form.instance.created_by = self.request.user
            self.object = form.save(commit=False)
            if labels.is_valid():
                labels.instance = self.object
                labels.save(commit=False)
            else:
                return super(SkosConceptCreate, self).form_invalid(form)
            if notes.is_valid():
                notes.instance = self.object
                notes.save(commit=False)
            else:
                return super(SkosConceptCreate, self).form_invalid(form)
            if sources.is_valid():
                sources.instance = self.object
                sources.save(commit=False)
            else:
                return super(SkosConceptCreate, self).form_invalid(form)
            self.object = form.save()
            labels.save()
            notes.save()
            sources.save()
        return super(SkosConceptCreate, self).form_valid(form)

    def get_initial(self):
        initial = super(SkosConceptCreate, self).get_initial()
        if self.request.GET.get('scheme'):
            initial['scheme'] = SkosConceptScheme.objects.get(pk=self.request.GET.get('scheme'))
        if self.request.GET.get('collection'):
            initial['collection'] = SkosCollection.objects.get(pk=self.request.GET.get('collection'))
        return initial

    def get_success_url(self):
        return reverse_lazy('vocabs:skosconcept_detail', kwargs={'pk': self.object.pk})

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(SkosConceptCreate, self).dispatch(*args, **kwargs)


class SkosConceptUpdate(BaseUpdateView):
    model = SkosConcept
    form_class = SkosConceptForm
    permission_required = (
        'view_skosconcept',
        'change_skosconcept',
        'delete_skosconcept',
    )
    success_url = None

    def get_context_data(self, **kwargs):
        data = super(SkosConceptUpdate, self).get_context_data(**kwargs)
        if self.request.POST:
            data['labels'] = ConceptLabelFormSet(self.request.POST, instance=self.object)
            data['notes'] = ConceptNoteFormSet(self.request.POST, instance=self.object)
            data['sources'] = ConceptSourceFormSet(self.request.POST, instance=self.object)
        else:
            data['labels'] = ConceptLabelFormSet(instance=self.object)
            data['notes'] = ConceptNoteFormSet(instance=self.object)
            data['sources'] = ConceptSourceFormSet(instance=self.object)
        return data

    def form_valid(self, form):
        context = self.get_context_data()
        labels = context['labels']
        notes = context['notes']
        sources = context['sources']
        with transaction.atomic():
            if labels.is_valid():
                labels.instance = self.object
                labels.save()
            else:
                return super(SkosConceptUpdate, self).form_invalid(form)
            if notes.is_valid():
                notes.instance = self.object
                notes.save()
            else:
                return super(SkosConceptUpdate, self).form_invalid(form)
            if sources.is_valid():
                sources.instance = self.object
                sources.save()
            else:
                return super(SkosConceptUpdate, self).form_invalid(form)
        return super(SkosConceptUpdate, self).form_valid(form)

    def get_success_url(self):
        return reverse_lazy('vocabs:skosconcept_detail', kwargs={'pk': self.object.pk})

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(SkosConceptUpdate, self).dispatch(*args, **kwargs)


class SkosConceptDelete(DeleteView):
    model = SkosConcept
    template_name = 'webpage/confirm_delete.html'
    success_url = reverse_lazy('vocabs:browse_vocabs')

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(SkosConceptDelete, self).dispatch(*args, **kwargs)


###################################################
# SkosConcepts download as one ConceptScheme
###################################################

class SkosConceptDL(GenericListView):
    model = SkosConcept
    table_class = SkosConceptTable
    filter_class = SkosConceptListFilter
    formhelper_class = SkosConceptFormHelper

    def render_to_response(self, context):
        timestamp = datetime.datetime.fromtimestamp(time.time()).strftime('%Y-%m-%d-%H-%M-%S')
        response = HttpResponse(content_type='application/xml; charset=utf-8')
        filename = "download_{}".format(timestamp)
        get_format = self.request.GET.get('format', default='pretty-xml')
        if get_format == 'turtle':
            response['Content-Disposition'] = 'attachment; filename="{}.ttl"'.format(filename)
        else:
            response['Content-Disposition'] = 'attachment; filename="{}.rdf"'.format(filename)
        g = graph_construct_qs(self.get_queryset())
        result = g.serialize(destination=response, format=get_format)
        return response


###################################################
# SKOS vocabulary upload
###################################################

@login_required
def file_upload(request):
    if request.method == 'POST':
        form = UploadFileForm(request.POST, request.FILES)
        if form.is_valid():
            file = request.FILES['file']
            file_format = file.name.split('.')[-1]
            if file_format in ['ttl', 'rdf']:
                if file_format == 'ttl':
                    skos_vocab = SkosImporter(file=file, file_format="ttl", language=form.cleaned_data['language'])
                else:
                    skos_vocab = SkosImporter(file=file, language=form.cleaned_data['language'])
                try:
                    skos_vocab.upload_data(user=request.user.username)
                    return redirect('vocabs:browse_schemes')
                except Exception as error:
                    messages.error(request, error)
            else:
                messages.error(request, "Upload rdf or ttl file")
    else:
        form = UploadFileForm()
    return render(request, 'vocabs/upload.html', {'form': form})



# Generated by Django 3.1.5 on 2021-01-05 13:12

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import mptt.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SkosCollection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Collection label or name', max_length=300, verbose_name='skos:prefLabel')),
                ('label_lang', models.CharField(blank=True, default='en', help_text='Language of preferred label given above', max_length=3, verbose_name='skos:prefLabel language')),
                ('creator', models.TextField(blank=True, help_text='Person or organisation that created this collection<br>If more than one list all using a semicolon ;', verbose_name='dc:creator')),
                ('contributor', models.TextField(blank=True, help_text='Person or organisation that made contributions to the collection<br>If more than one list all using a semicolon ;', verbose_name='dc:contributor')),
                ('legacy_id', models.CharField(blank=True, max_length=200)),
                ('date_created', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('date_modified', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='skos_collection_created', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Collection',
                'ordering': ['id'],
            },
        ),
        migrations.CreateModel(
            name='SkosConceptScheme',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Title  for new concept scheme', max_length=300, verbose_name='dc:title')),
                ('title_lang', models.CharField(blank=True, default='en', help_text='Language of title given above', max_length=3, verbose_name='dc:title language')),
                ('identifier', models.URLField(blank=True, help_text='URI to unambiguously identify current Concept Scheme')),
                ('creator', models.TextField(blank=True, help_text='Person or organisation primarily responsible for making current concept scheme<br>If more than one list all using a semicolon ;', verbose_name='dc:creator')),
                ('contributor', models.TextField(blank=True, help_text='Person or organisation that made contributions to the vocabulary<br>If more than one list all using a semicolon ;', verbose_name='dc:contributor')),
                ('language', models.TextField(blank=True, help_text='Language(s) used in concept scheme<br>If more than one list all using a semicolon ;', verbose_name='dc:language')),
                ('subject', models.TextField(blank=True, help_text='The subject of the vocabulary<br>If more than one list all using a semicolon ;', verbose_name='dc:subject')),
                ('version', models.CharField(blank=True, help_text='Current version', max_length=300)),
                ('publisher', models.CharField(blank=True, help_text='Organisation responsible for making the vocabulary available', max_length=300, verbose_name='dc:publisher')),
                ('license', models.CharField(blank=True, help_text='Information about license applied to the vocabulary', max_length=300, verbose_name='dct:license')),
                ('owner', models.CharField(blank=True, help_text='Person or organisation that owns the rights for the vocabulary', max_length=300)),
                ('relation', models.URLField(blank=True, help_text='Related resource or project<br>E.g. in case of relation to a project, add link to a project website', verbose_name='dc:relation')),
                ('coverage', models.TextField(blank=True, help_text='Spatial or temporal frame that the vocabulary relates to<br>If more than one list all using a semicolon ;', verbose_name='dc:coverage')),
                ('legacy_id', models.CharField(blank=True, max_length=200)),
                ('date_created', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('date_modified', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('date_issued', models.DateField(blank=True, help_text='Date of official publication of this concept scheme', null=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='skos_cs_created', to=settings.AUTH_USER_MODEL)),
                ('curator', models.ManyToManyField(blank=True, help_text='The selected user(s) will be able to view and edit this Concept Scheme', related_name='skos_cs_curated', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Concept Scheme',
                'ordering': ['id'],
            },
        ),
        migrations.CreateModel(
            name='SkosConcept',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('pref_label', models.CharField(help_text='Preferred label for concept', max_length=300, verbose_name='skos:prefLabel')),
                ('pref_label_lang', models.CharField(blank=True, default='en', help_text='Language of preferred label given above', max_length=3, verbose_name='skos:prefLabel language')),
                ('top_concept', models.BooleanField(help_text='Is this concept a top concept of concept scheme?', null=True)),
                ('notation', models.CharField(blank=True, help_text='A notation is a unique string used        to identify the concept in current vocabulary', max_length=300, verbose_name='skos:notation')),
                ('related', models.TextField(blank=True, help_text='An associative relationship between two concepts', verbose_name='skos:related')),
                ('broad_match', models.TextField(blank=True, help_text='External concept with a broader meaning', verbose_name='skos:broadMatch')),
                ('narrow_match', models.TextField(blank=True, help_text='External concept with a narrower meaning', verbose_name='skos:narrowMatch')),
                ('exact_match', models.TextField(blank=True, help_text='External concept that can be used interchangeably and has the exact same meaning', verbose_name='skos:exactMatch')),
                ('related_match', models.TextField(blank=True, help_text='External concept that has an associative relationship with this concept', verbose_name='skos:relatedMatch')),
                ('close_match', models.TextField(blank=True, help_text='External concept that has a similar meaning', verbose_name='skos:closeMatch')),
                ('legacy_id', models.CharField(blank=True, max_length=200)),
                ('creator', models.TextField(blank=True, help_text='Person or organisation that created this concept<br>If more than one list all using a semicolon ;', verbose_name='dc:creator')),
                ('contributor', models.TextField(blank=True, help_text='Person or organisation that made contributions to this concept<br>If more than one list all using a semicolon ;', verbose_name='dc:contributor')),
                ('needs_review', models.BooleanField(help_text='Check if this concept needs to be reviewed', null=True)),
                ('date_created', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='dct:created')),
                ('date_modified', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='dct:modified')),
                ('lft', models.PositiveIntegerField(editable=False)),
                ('rght', models.PositiveIntegerField(editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(editable=False)),
                ('broader_concept', mptt.fields.TreeForeignKey(blank=True, help_text='Concept with a broader meaning that this concept inherits from', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='narrower_concepts', to='vocabs.skosconcept', verbose_name='skos:broader')),
                ('collection', models.ManyToManyField(blank=True, help_text='Collection that this concept is a member of', related_name='has_members', to='vocabs.SkosCollection', verbose_name='member of skos:Collection')),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='skos_concept_created', to=settings.AUTH_USER_MODEL)),
                ('scheme', models.ForeignKey(help_text='Concept scheme to which this concept belongs', on_delete=django.db.models.deletion.CASCADE, related_name='has_concepts', to='vocabs.skosconceptscheme', verbose_name='skos:inScheme')),
            ],
            options={
                'verbose_name': 'Concept',
            },
        ),
        migrations.AddField(
            model_name='skoscollection',
            name='scheme',
            field=models.ForeignKey(help_text='Concept scheme that this collection belongs to', on_delete=django.db.models.deletion.CASCADE, related_name='has_collections', to='vocabs.skosconceptscheme', verbose_name='skos:ConceptScheme'),
        ),
        migrations.CreateModel(
            name='ConceptSource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField(help_text="Verbal description of the concept's source", verbose_name='dc:source')),
                ('language', models.CharField(help_text='Language of source given above', max_length=3, verbose_name='language')),
                ('concept', models.ForeignKey(help_text='Which Skos:Concept current source belongs to', on_delete=django.db.models.deletion.CASCADE, related_name='has_sources', to='vocabs.skosconcept', verbose_name='skos:Concept')),
            ],
        ),
        migrations.CreateModel(
            name='ConceptSchemeTitle',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Other title for new concept scheme', max_length=500, verbose_name='dc:title')),
                ('language', models.CharField(help_text='Language of title given above', max_length=3, verbose_name='dc:title language')),
                ('concept_scheme', models.ForeignKey(help_text='Which Skos:ConceptScheme current Title belongs to', on_delete=django.db.models.deletion.CASCADE, related_name='has_titles', to='vocabs.skosconceptscheme', verbose_name='skos:ConceptScheme')),
            ],
        ),
        migrations.CreateModel(
            name='ConceptSchemeSource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField(help_text="Verbal description of a concept scheme's source", verbose_name='dc:source')),
                ('language', models.CharField(help_text='Language of source given above', max_length=3, verbose_name='dc:source language')),
                ('concept_scheme', models.ForeignKey(help_text='Which Skos:ConceptScheme current source belongs to', on_delete=django.db.models.deletion.CASCADE, related_name='has_sources', to='vocabs.skosconceptscheme', verbose_name='skos:ConceptScheme')),
            ],
        ),
        migrations.CreateModel(
            name='ConceptSchemeDescription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField(help_text='Description of concept scheme', verbose_name='dc:description')),
                ('language', models.CharField(help_text='Language of description given above', max_length=3, verbose_name='dc:description language')),
                ('concept_scheme', models.ForeignKey(help_text='Which Skos:ConceptScheme current Description belongs to', on_delete=django.db.models.deletion.CASCADE, related_name='has_descriptions', to='vocabs.skosconceptscheme', verbose_name='skos:ConceptScheme')),
            ],
        ),
        migrations.CreateModel(
            name='ConceptNote',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField(help_text='Provide some information about this concept', verbose_name='documentary note')),
                ('language', models.CharField(help_text='Language of note given above', max_length=3, verbose_name='language')),
                ('note_type', models.CharField(choices=[('note', 'note'), ('scopeNote', 'scopeNote'), ('changeNote', 'changeNote'), ('editorialNote', 'editorialNote'), ('historyNote', 'historyNote'), ('definition', 'definition'), ('example', 'example')], default='note', help_text='Choose note type', max_length=15, verbose_name='note type')),
                ('concept', models.ForeignKey(help_text='Which Skos:Concept current documentary note belongs to', on_delete=django.db.models.deletion.CASCADE, related_name='has_notes', to='vocabs.skosconcept', verbose_name='skos:Concept')),
            ],
        ),
        migrations.CreateModel(
            name='ConceptLabel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Other label for this concept', max_length=500, verbose_name='label')),
                ('language', models.CharField(help_text='Language of label given above', max_length=3, verbose_name='language')),
                ('label_type', models.CharField(choices=[('prefLabel', 'prefLabel'), ('altLabel', 'altLabel'), ('hiddenLabel', 'hiddenLabel')], default='altLabel', help_text='Choose label type', max_length=12, verbose_name='label type')),
                ('concept', models.ForeignKey(help_text='Which Skos:Concept current label belongs to', on_delete=django.db.models.deletion.CASCADE, related_name='has_labels', to='vocabs.skosconcept', verbose_name='skos:Concept')),
            ],
        ),
        migrations.CreateModel(
            name='CollectionSource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField(help_text="Verbal description of the collection's source", verbose_name='dc:source')),
                ('language', models.CharField(help_text='Language of source given above', max_length=3, verbose_name='language')),
                ('collection', models.ForeignKey(help_text='Which Skos:Collection current source belongs to', on_delete=django.db.models.deletion.CASCADE, related_name='has_sources', to='vocabs.skoscollection', verbose_name='skos:Collection')),
            ],
        ),
        migrations.CreateModel(
            name='CollectionNote',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField(help_text='Provide some information about this collection', verbose_name='documentary note')),
                ('language', models.CharField(help_text='Language of note given above', max_length=3, verbose_name='language')),
                ('note_type', models.CharField(choices=[('note', 'note'), ('scopeNote', 'scopeNote'), ('changeNote', 'changeNote'), ('editorialNote', 'editorialNote'), ('historyNote', 'historyNote'), ('definition', 'definition'), ('example', 'example')], default='note', help_text='Choose note type', max_length=15, verbose_name='note type')),
                ('collection', models.ForeignKey(help_text='Which Skos:Collection current documentary note belongs to', on_delete=django.db.models.deletion.CASCADE, related_name='has_notes', to='vocabs.skoscollection', verbose_name='skos:Collection')),
            ],
        ),
        migrations.CreateModel(
            name='CollectionLabel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Other label for this collection', max_length=500, verbose_name='label')),
                ('language', models.CharField(help_text='Language of label given above', max_length=3, verbose_name='language')),
                ('label_type', models.CharField(choices=[('prefLabel', 'prefLabel'), ('altLabel', 'altLabel'), ('hiddenLabel', 'hiddenLabel')], default='altLabel', help_text='Choose label type', max_length=12, verbose_name='label type')),
                ('collection', models.ForeignKey(help_text='Which Skos:Collection current label belongs to', on_delete=django.db.models.deletion.CASCADE, related_name='has_labels', to='vocabs.skoscollection', verbose_name='skos:Collection')),
            ],
        ),
    ]



from rest_framework import serializers
from .models import *


class SkosConceptSchemeSerializer(serializers.HyperlinkedModelSerializer):
    has_concepts = serializers.HyperlinkedRelatedField(many=True, read_only=True, view_name='skosconcept-detail')
    created_by = serializers.CharField(read_only=True)
    curator = serializers.StringRelatedField(many=True)

    class Meta:
        model = SkosConceptScheme
        fields = '__all__'


class SkosCollectionSerializer(serializers.HyperlinkedModelSerializer):
    has_members = serializers.HyperlinkedRelatedField(many=True, read_only=True, view_name='skosconcept-detail')
    created_by = serializers.CharField(read_only=True)

    class Meta:
        model = SkosCollection
        fields = '__all__'


class SkosConceptSerializer(serializers.HyperlinkedModelSerializer):
    created_by = serializers.CharField(read_only=True)

    class Meta:
        model = SkosConcept
        fields = (
            'id', 'url',
            'pref_label', 'pref_label_lang',
            'scheme', 'collection',
            'broader_concept', 'narrower_concepts',
            'notation', 'related',
            'broad_match', 'narrow_match',
            'exact_match', 'related_match',
            'close_match', 'legacy_id',
            'creator', 'contributor',
            'date_created', 'date_modified',
            'created_by',
        )



import requests
import json

from dal import autocomplete
from django.contrib.auth.models import User
from mptt.settings import DEFAULT_LEVEL_INDICATOR

from . endpoints import *
from . models import SkosConcept, SkosConceptScheme, SkosCollection


# Global autocomplete for external concepts ################


def global_autocomplete(request, endpoint):
    choices = []
    q = request.GET.get('q')
    headers = {'accept': 'application/json'}
    ac_instance = ENDPOINT.get(endpoint, DbpediaAC())
    if ac_instance.__class__.__name__.startswith('Fish'):
        scheme = ac_instance.scheme_dict.get(endpoint, 'FISH Event Types Thesaurus')
        r = requests.get(ac_instance.get_url(), headers=headers,
                         params=ac_instance.payload(scheme=scheme, q=q))
    else:
        r = requests.get(ac_instance.get_url(), headers=headers,
                         params=ac_instance.payload(q=q))
    response = json.loads(r.content.decode('utf-8'))
    choices = ac_instance.parse_response(response=response)
    return choices


###########################################################################


class ExternalLinkAC(autocomplete.Select2ListView):

    def get_list(self):
        choices = []
        endpoint = self.forwarded.get('endpoint', None)
        global_ac = global_autocomplete(self.request, endpoint=endpoint)
        return global_ac


class SkosConceptAC(autocomplete.Select2QuerySetView):

    def get_result_label(self, item):
        level_indicator = DEFAULT_LEVEL_INDICATOR * item.level
        return level_indicator + ' ' + str(item)

    def get_queryset(self):
        qs = SkosConcept.objects.all()
        scheme = self.forwarded.get('scheme', None)
        if scheme:
            qs = qs.filter(scheme=scheme)
        if self.q:
            qs = qs.filter(pref_label__icontains=self.q)
        return qs


class SkosConceptExternalMatchAC(autocomplete.Select2QuerySetView):

    def get_result_label(self, item):
        level_indicator = DEFAULT_LEVEL_INDICATOR * item.level
        return level_indicator + ' ' + str(item)

    def get_queryset(self):
        qs = SkosConcept.objects.all()
        scheme = self.forwarded.get('scheme', None)
        if scheme:
            qs = qs.exclude(scheme=scheme)
        if self.q:
            qs = qs.filter(pref_label__icontains=self.q)
        return qs


class SkosConceptSchemeAC(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        qs = SkosConceptScheme.objects.all()
        if self.q:
            qs = qs.filter(title__icontains=self.q)

        return qs


class SkosCollectionAC(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        qs = SkosCollection.objects.all()
        scheme = self.forwarded.get('scheme', None)
        if scheme:
            qs = qs.filter(scheme=scheme)

        if self.q:
            qs = qs.filter(name__icontains=self.q)

        return qs


class UserAC(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        qs = User.objects.exclude(username=self.request.user)
        if self.q:
            qs = qs.filter(username__icontains=self.q)

        return qs



from django.core.exceptions import ValidationError
from django.db.utils import IntegrityError
from django.test import TestCase
from django.contrib.auth.models import User

from .constants import *
from ..models import SkosConceptScheme, SkosCollection, SkosConcept


class ConceptSchemeTest(TestCase):
    """ Test module for SkosConceptScheme model """

    def setUp(self):
        self.user = User.objects.create_user(**USER)
        self.concept_scheme = SkosConceptScheme.objects.create(**concept_scheme(self.user))

    def test_create(self):
        concept_scheme = SkosConceptScheme.objects.get(title="Test Concept Scheme")
        self.assertEqual(concept_scheme.title, "Test Concept Scheme")
        self.assertEqual(len(SkosConceptScheme.objects.all()), 1)


class CollectionTest(TestCase):
    """ Test module for SkosCollection model """

    def setUp(self):
        self.user = User.objects.create_user(**USER)
        self.concept_scheme = SkosConceptScheme.objects.create(**concept_scheme(self.user))
        self.collection = SkosCollection.objects.create(**collection(self.concept_scheme, self.user))

    def test_create(self):
        collection = SkosCollection.objects.get(name="Test Collection")
        self.assertEqual(collection.name, "Test Collection")
        self.assertEqual(len(SkosCollection.objects.all()), 1)


class ConceptTest(TestCase):
    """ Test module for SkosConcept model """

    def setUp(self):
        self.user = User.objects.create_user(**USER)
        self.concept_scheme = SkosConceptScheme.objects.create(**concept_scheme(self.user))
        self.collection = SkosCollection.objects.create(**collection(self.concept_scheme, self.user))
        self.concept = SkosConcept.objects.create(**concept(self.concept_scheme, "Concept 1", self.user))

    def test_create(self):
        concept_one = SkosConcept.objects.get(pref_label="Concept 1")
        self.assertEqual(concept_one.pref_label, "Concept 1")
        self.assertEqual(len(SkosConcept.objects.all()), 1)



from django.conf.urls import url
from . import views


app_name = 'vocabs'

urlpatterns = [
    url(r'^concepts/$', views.SkosConceptListView.as_view(), name='browse_vocabs'),
    url(r'^concepts/(?P<pk>[0-9]+)$', views.SkosConceptDetailView.as_view(), name='skosconcept_detail'),
    url(r'^concepts/create/$', views.SkosConceptCreate.as_view(), name='skosconcept_create'),
    url(r'^concepts/update/(?P<pk>[0-9]+)$', views.SkosConceptUpdate.as_view(), name='skosconcept_update'),
    url(r'^concepts/delete/(?P<pk>[0-9]+)$', views.SkosConceptDelete.as_view(), name='skosconcept_delete'),
    url(r'^scheme/$', views.SkosConceptSchemeListView.as_view(), name='browse_schemes'),
    url(
        r'^scheme/(?P<pk>[0-9]+)$', views.SkosConceptSchemeDetailView.as_view(),
        name='skosconceptscheme_detail'),
    url(
        r'^scheme/create/$', views.SkosConceptSchemeCreate.as_view(),
        name='skosconceptscheme_create'),
    url(
        r'^scheme/update/(?P<pk>[0-9]+)$', views.SkosConceptSchemeUpdate.as_view(),
        name='skosconceptscheme_update'),
    url(
        r'^scheme/delete/(?P<pk>[0-9]+)$',
        views.SkosConceptSchemeDelete.as_view(),
        name='skosconceptscheme_delete',
    ),
    url(
        r'^vocabs-download/$', views.SkosConceptDL.as_view(),
        name='vocabs-download'),
    url(
        r'^collection/$', views.SkosCollectionListView.as_view(),
        name='browse_skoscollections'),
    url(
        r'^collection/(?P<pk>[0-9]+)$', views.SkosCollectionDetailView.as_view(),
        name='skoscollection_detail'),
    url(
        r'^collection/create/$', views.SkosCollectionCreate.as_view(),
        name='skoscollection_create'),
    url(
        r'^collection/update/(?P<pk>[0-9]+)$', views.SkosCollectionUpdate.as_view(),
        name='skoscollection_update'),
    url(
        r'^collection/delete/(?P<pk>[0-9]+)$', views.SkosCollectionDelete.as_view(),
        name='skoscollection_delete',
    ),
    url(r'^import/$', views.file_upload, name='import'),
]



import django_tables2 as tables
from django_tables2.utils import A
from vocabs.models import *


class SkosConceptSchemeTable(tables.Table):
    title = tables.LinkColumn('vocabs:skosconceptscheme_detail', args=[A('pk')])

    class Meta:
        model = SkosConceptScheme
        sequence = ['id', 'title']
        attrs = {"class": "table table-hover table-striped table-condensed"}


class SkosCollectionTable(tables.Table):
    name = tables.LinkColumn('vocabs:skoscollection_detail', args=[A('pk')])

    class Meta:
        model = SkosCollection
        sequence = ['id', 'name', 'scheme']
        attrs = {"class": "table table-hover table-striped table-condensed"}


class SkosConceptTable(tables.Table):
    pref_label = tables.LinkColumn('vocabs:skosconcept_detail', args=[A('pk')])

    class Meta:
        model = SkosConcept
        sequence = ['id', 'pref_label', 'scheme']
        attrs = {"class": "table table-hover table-striped table-condensed"}



import os

from django.test import TestCase
from django.contrib.auth.models import User
from rdflib import Graph

from .constants import USER
from ..models import SkosConceptScheme, SkosCollection, SkosConcept
from ..skos_import import SkosImporter
from ..rdf_utils import graph_construct_qs


EXAMPLE_SKOS_IMPORT = os.path.join(os.path.dirname(__file__), "example_skos_import.rdf")
EXAMPLE_SKOS_EXPORT = os.path.join(os.path.dirname(__file__), "example_skos_export.rdf")


class TestSkosImport(TestCase):
    """ Test module for SKOS import functionality. """

    def setUp(self) -> None:
        self.user = User.objects.create_user(**USER)
        skos_vocab = SkosImporter(file=EXAMPLE_SKOS_IMPORT, file_format="xml", language="en")
        self.concept_scheme = skos_vocab.parse_triples()
        skos_vocab.upload_data(self.user)

    def test_parsing_to_graph(self):
        self.assertEqual(type(self.concept_scheme), dict)
        self.assertEqual(self.concept_scheme["identifier"], "https://vocabs.acdh.oeaw.ac.at/dhataxonomy/ConceptScheme")
        self.assertEqual(type(self.concept_scheme["title"]), list)
        self.assertEqual(self.concept_scheme["title"][0]["title"], "DHA Taxonomy")
        self.assertEqual(self.concept_scheme["title"][0]["lang"], "en")

    def test_uploading_data(self):
        self.assertEqual(len(SkosConceptScheme.objects.all()), 1)
        self.assertEqual(len(SkosCollection.objects.all()), 6)
        self.assertEqual(len(SkosConcept.objects.all()), 114)


class TestSkosExport(TestCase):
    """ Test module for SKOS export functionality. """

    def setUp(self) -> None:
        self.user = User.objects.create_user(**USER)
        skos_vocab = SkosImporter(file=EXAMPLE_SKOS_EXPORT, file_format="xml", language="en")
        self.concept_scheme = skos_vocab.parse_triples()
        skos_vocab.upload_data(self.user)

    def test_skos_export(self):
        g = graph_construct_qs(SkosConcept.objects.all())
        self.assertEqual(Graph, type(g))



import reversion

from django.conf import settings
from django.db import models
from django.urls import reverse
from django.utils import timezone
from django.utils.text import slugify
from django.contrib.auth.models import User
from django.dispatch import receiver
from mptt.models import MPTTModel, TreeForeignKey


try:
    notation_for_uri = settings.VOCABS_SETTINGS['notation_for_uri']
except KeyError:
    notation_for_uri = False


DEFAULT_URI = "https://vocabs.acdh.oeaw.ac.at/"

try:
    DEFAULT_NAMESPACE = settings.VOCABS_SETTINGS['default_nsgg']
except KeyError:
    DEFAULT_NAMESPACE = "https://vocabs.acdh.oeaw.ac.at/provide-some-namespace"

try:
    DEFAULT_PREFIX = settings.VOCABS_SETTINGS['default_prefix']
except KeyError:
    DEFAULT_PREFIX = "provideSome"

try:
    DEFAULT_LANG = settings.VOCABS_SETTINGS['default_lang']
except KeyError:
    DEFAULT_LANG = "en"

LABEL_TYPES = (
    ('prefLabel', 'prefLabel'),
    ('altLabel', 'altLabel'),
    ('hiddenLabel', 'hiddenLabel'),
)

NOTE_TYPES = (
    ('note', 'note'),
    ('scopeNote', 'scopeNote'),
    ('changeNote', 'changeNote'),
    ('editorialNote', 'editorialNote'),
    ('historyNote', 'historyNote'),
    ('definition', 'definition'),
    ('example', 'example'),
)


######################################################################
#
# SkosConceptScheme
#
######################################################################

@reversion.register()
class SkosConceptScheme(models.Model):
    """
    A SKOS concept scheme can be viewed as an aggregation of one or more SKOS concepts.
    Semantic relationships (links) between those concepts
    may also be viewed as part of a concept scheme.

    Miles, Alistair, and Sean Bechhofer. "SKOS simple knowledge
    organization system reference. W3C recommendation (2009)."
    """
    title = models.CharField(
        max_length=300,
        help_text="Title  for new concept scheme",
        verbose_name="dc:title"
    )
    title_lang = models.CharField(
        max_length=3, blank=True,
        verbose_name="dc:title language", default=DEFAULT_LANG,
        help_text="Language of title given above"
    )
    identifier = models.URLField(
        blank=True, help_text="URI to unambiguously identify current Concept Scheme"
    )
    creator = models.TextField(
        blank=True, verbose_name="dc:creator",
        help_text="Person or organisation primarily responsible for making current concept scheme<br>"
                  "If more than one list all using a semicolon ;"
    )
    contributor = models.TextField(
        blank=True, verbose_name="dc:contributor",
        help_text="Person or organisation that made contributions to the vocabulary<br>"
                  "If more than one list all using a semicolon ;"
    )
    language = models.TextField(
        blank=True, verbose_name="dc:language",
        help_text="Language(s) used in concept scheme<br>"
                  "If more than one list all using a semicolon ;"
    )
    subject = models.TextField(
        blank=True, verbose_name="dc:subject",
        help_text="The subject of the vocabulary<br>"
                  "If more than one list all using a semicolon ;"
    )
    version = models.CharField(
        max_length=300, blank=True,
        help_text="Current version"
    )
    publisher = models.CharField(
        max_length=300, blank=True,
        help_text="Organisation responsible for making the vocabulary available",
        verbose_name="dc:publisher"
    )
    license = models.CharField(
        max_length=300, blank=True,
        verbose_name="dct:license",
        help_text="Information about license applied to the vocabulary"
    )
    owner = models.CharField(
        max_length=300, blank=True,
        help_text="Person or organisation that owns the rights for the vocabulary"
    )
    relation = models.URLField(
        blank=True, verbose_name="dc:relation",
        help_text="Related resource or project<br>"
                  "E.g. in case of relation to a project, add link to a project website"
    )
    coverage = models.TextField(
        blank=True, verbose_name="dc:coverage",
        help_text="Spatial or temporal frame that the vocabulary relates to<br>"
                  "If more than one list all using a semicolon ;"
    )
    legacy_id = models.CharField(
        max_length=200, blank=True
    )
    date_created = models.DateTimeField(
        editable=False, default=timezone.now
    )
    date_modified = models.DateTimeField(
        editable=False, default=timezone.now
    )
    date_issued = models.DateField(
        blank=True, null=True,
        help_text="Date of official publication of this concept scheme"
    )
    created_by = models.ForeignKey(
        User, related_name="skos_cs_created",
        blank=True, null=True,
        on_delete=models.SET_NULL
    )
    curator = models.ManyToManyField(
        User, related_name="skos_cs_curated",
        blank=True,
        help_text="The selected user(s) will be able to view and edit this Concept Scheme"
    )

    class Meta:
        ordering = ['id']
        verbose_name = 'Concept Scheme'

    def save(self, *args, **kwargs):
        if not self.id:
            self.date_created = timezone.now()
        self.date_modified = timezone.now()

        if not self.identifier:
            self.identifier = DEFAULT_URI + slugify(self.title, allow_unicode=True)
        super(SkosConceptScheme, self).save(*args, **kwargs)

    def creator_as_list(self):
        return self.creator.split(';')

    def contributor_as_list(self):
        return self.contributor.split(';')

    def language_as_list(self):
        return self.language.split(';')

    def subject_as_list(self):
        return self.subject.split(';')

    def coverage_as_list(self):
        return self.coverage.split(';')

    @classmethod
    def get_listview_url(self):
        return reverse('vocabs:browse_schemes')

    @classmethod
    def get_createview_url(self):
        return reverse('vocabs:skosconceptscheme_create')

    def get_absolute_url(self):
        return reverse('vocabs:skosconceptscheme_detail', kwargs={'pk': self.id})

    def get_next(self):
        next = SkosConceptScheme.objects.filter(id__gt=self.id)
        if next:
            return next.first().id
        return False

    def get_prev(self):
        prev = SkosConceptScheme.objects.filter(id__lt=self.id).order_by('-id')
        if prev:
            return prev.first().id
        return False

    def __str__(self):
        return self.title


######################################################################
#   Classes  to store titles and descriptions for ConceptScheme
######################################################################

class ConceptSchemeTitle(models.Model):
    """
    A Class for ConceptScheme titles in other languages.

    """
    concept_scheme = models.ForeignKey(
        SkosConceptScheme,
        related_name="has_titles",
        verbose_name="skos:ConceptScheme",
        help_text="Which Skos:ConceptScheme current Title belongs to",
        on_delete=models.CASCADE
    )
    name = models.CharField(
        max_length=500, verbose_name="dc:title",
        help_text="Other title for new concept scheme"
    )
    language = models.CharField(
        max_length=3, verbose_name="dc:title language",
        help_text="Language of title given above"
    )

    def __str__(self):
        return "{}".format(self.name)


class ConceptSchemeDescription(models.Model):
    """
    A Class for ConceptScheme descriptions in other languages.

    """
    concept_scheme = models.ForeignKey(
        SkosConceptScheme,
        related_name="has_descriptions",
        verbose_name="skos:ConceptScheme",
        help_text="Which Skos:ConceptScheme current Description belongs to",
        on_delete=models.CASCADE
    )
    name = models.TextField(
        verbose_name="dc:description",
        help_text="Description of concept scheme"
    )
    language = models.CharField(
        max_length=3, verbose_name="dc:description language",
        help_text="Language of description given above"
    )

    def __str__(self):
        return self.name


class ConceptSchemeSource(models.Model):
    """
    A Class for ConceptScheme source information.

    """
    concept_scheme = models.ForeignKey(
        SkosConceptScheme,
        related_name="has_sources",
        verbose_name="skos:ConceptScheme",
        help_text="Which Skos:ConceptScheme current source belongs to",
        on_delete=models.CASCADE
    )
    name = models.TextField(
        verbose_name="dc:source",
        help_text="Verbal description of a concept scheme's source"
    )
    language = models.CharField(
        max_length=3, verbose_name="dc:source language",
        help_text="Language of source given above"
    )

    def __str__(self):
        return "{}".format(self.name)


######################################################################
#
# SkosCollection
#
######################################################################

@reversion.register()
class SkosCollection(models.Model):
    """
    SKOS collections are labeled and/or ordered groups of SKOS concepts.
    Collections are useful where a group of concepts shares something in common,
    and it is convenient to group them under a common label, or
    where some concepts can be placed in a meaningful order.

    Miles, Alistair, and Sean Bechhofer. "SKOS simple knowledge
    organization system reference. W3C recommendation (2009)."

    """
    name = models.CharField(
        max_length=300, verbose_name="skos:prefLabel",
        help_text="Collection label or name"
    )
    label_lang = models.CharField(
        max_length=3, blank=True,
        default=DEFAULT_LANG,
        verbose_name="skos:prefLabel language",
        help_text="Language of preferred label given above"
    )
    # relation to SkosConceptScheme to inherit all objects permissions
    scheme = models.ForeignKey(SkosConceptScheme,
                               related_name="has_collections",
                               verbose_name="skos:ConceptScheme",
                               help_text="Concept scheme that this collection belongs to",
                               on_delete=models.CASCADE
                               )
    creator = models.TextField(
        blank=True, verbose_name="dc:creator",
        help_text="Person or organisation that created this collection<br>"
                  "If more than one list all using a semicolon ;"
    )
    contributor = models.TextField(
        blank=True, verbose_name="dc:contributor",
        help_text="Person or organisation that made contributions to the collection<br>"
                  "If more than one list all using a semicolon ;"
    )
    legacy_id = models.CharField(
        max_length=200, blank=True
    )
    # meta autosaved fields
    date_created = models.DateTimeField(
        editable=False, default=timezone.now
    )
    date_modified = models.DateTimeField(
        editable=False, default=timezone.now
    )
    created_by = models.ForeignKey(
        User, related_name="skos_collection_created",
        blank=True, null=True,
        on_delete=models.SET_NULL
    )

    class Meta:
        ordering = ['id']
        verbose_name = 'Collection'

    def save(self, *args, **kwargs):
        if not self.id:
            self.date_created = timezone.now()
        self.date_modified = timezone.now()
        return super(SkosCollection, self).save(*args, **kwargs)

    @classmethod
    def get_listview_url(self):
        return reverse('vocabs:browse_skoscollections')

    @classmethod
    def get_createview_url(self):
        return reverse('vocabs:skoscollection_create')

    def get_absolute_url(self):
        return reverse('vocabs:skoscollection_detail', kwargs={'pk': self.id})

    def get_next(self):
        next = SkosCollection.objects.filter(id__gt=self.id)
        if next:
            return next.first().id
        return False

    def get_prev(self):
        prev = SkosCollection.objects.filter(id__lt=self.id).order_by('-id')
        if prev:
            return prev.first().id
        return False

    def __str__(self):
        return self.name

    def creator_as_list(self):
        return self.creator.split(';')

    def contributor_as_list(self):
        return self.contributor.split(';')


######################################################################
#   Classes  to store labels and notes for Collection
######################################################################


class CollectionLabel(models.Model):
    """
    A Class for Collection labels/names in other languages.

    """
    collection = models.ForeignKey(
        SkosCollection,
        related_name="has_labels",
        verbose_name="skos:Collection",
        help_text="Which Skos:Collection current label belongs to",
        on_delete=models.CASCADE
    )
    name = models.CharField(
        max_length=500, verbose_name="label",
        help_text="Other label for this collection"
    )
    language = models.CharField(
        max_length=3, verbose_name="language",
        help_text="Language of label given above"
    )
    label_type = models.CharField(
        choices=LABEL_TYPES, verbose_name="label type",
        default='altLabel', max_length=12,
        help_text="Choose label type"
    )

    def __str__(self):
        return "{}".format(self.name)


class CollectionNote(models.Model):
    """
    A Class for SKOS documentary notes that are used
    for general documentation pusposes.

    """
    collection = models.ForeignKey(
        SkosCollection,
        related_name="has_notes",
        verbose_name="skos:Collection",
        help_text="Which Skos:Collection current documentary note belongs to",
        on_delete=models.CASCADE
    )
    name = models.TextField(
        verbose_name="documentary note",
        help_text="Provide some information about this collection"
    )
    language = models.CharField(
        max_length=3, verbose_name="language",
        help_text="Language of note given above"
    )
    note_type = models.CharField(
        choices=NOTE_TYPES, verbose_name="note type",
        default='note', max_length=15,
        help_text="Choose note type"
    )

    def __str__(self):
        return "{}".format(self.name)


class CollectionSource(models.Model):
    """
    A Class for Collection source information.

    """
    collection = models.ForeignKey(
        SkosCollection,
        related_name="has_sources",
        verbose_name="skos:Collection",
        help_text="Which Skos:Collection current source belongs to",
        on_delete=models.CASCADE
    )
    name = models.TextField(
        verbose_name="dc:source",
        help_text="Verbal description of the collection's source"
    )
    language = models.CharField(
        max_length=3, verbose_name="language",
        help_text="Language of source given above"
    )

    def __str__(self):
        return "{}".format(self.name)


######################################################################
#
# SkosConcept
#
######################################################################

@reversion.register()
class SkosConcept(MPTTModel):
    """
    A SKOS concept can be viewed as an idea or notion; a unit of thought.
    However, what constitutes a unit of thought is subjective,
    and this definition is meant to be suggestive, rather than restrictive.

    Miles, Alistair, and Sean Bechhofer. "SKOS simple knowledge
    organization system reference. W3C recommendation (2009)."
    """
    pref_label = models.CharField(
        max_length=300,
        verbose_name="skos:prefLabel",
        help_text="Preferred label for concept"
    )
    pref_label_lang = models.CharField(
        max_length=3, blank=True,
        verbose_name="skos:prefLabel language",
        help_text="Language of preferred label given above",
        default=DEFAULT_LANG
    )
    # relation to SkosConceptScheme to inherit all objects permissions
    scheme = models.ForeignKey(
        SkosConceptScheme,
        verbose_name="skos:inScheme",
        related_name="has_concepts",
        on_delete=models.CASCADE,
        help_text="Concept scheme to which this concept belongs"
    )
    top_concept = models.BooleanField(
        null=True,
        help_text="Is this concept a top concept of concept scheme?"
    )
    collection = models.ManyToManyField(
        SkosCollection, blank=True,
        verbose_name="member of skos:Collection",
        help_text="Collection that this concept is a member of",
        related_name="has_members",
    )
    notation = models.CharField(
        max_length=300, blank=True,
        verbose_name="skos:notation",
        help_text="A notation is a unique string used\
        to identify the concept in current vocabulary"
    )
    broader_concept = TreeForeignKey(
        'self',
        verbose_name="skos:broader",
        blank=True, null=True, on_delete=models.CASCADE,
        related_name="narrower_concepts",
        help_text="Concept with a broader meaning that this concept inherits from"
    )
    ################# semantic relationships via autocomplete #################
    related = models.TextField(
        blank=True, verbose_name="skos:related",
        help_text="An associative relationship between two concepts"
    )
    broad_match = models.TextField(
        blank=True, verbose_name="skos:broadMatch",
        help_text="External concept with a broader meaning"
    )
    narrow_match = models.TextField(
        blank=True, verbose_name="skos:narrowMatch",
        help_text="External concept with a narrower meaning"
    )
    exact_match = models.TextField(
        blank=True, verbose_name="skos:exactMatch",
        help_text="External concept that can be used interchangeably and has the exact same meaning"
    )
    related_match = models.TextField(
        blank=True, verbose_name="skos:relatedMatch",
        help_text="External concept that has an associative relationship with this concept"
    )
    close_match = models.TextField(
        blank=True, verbose_name="skos:closeMatch",
        help_text="External concept that has a similar meaning"
    )
    ###########################################################################
    # if using legacy_id as URI change it for URLField
    legacy_id = models.CharField(max_length=200, blank=True)
    creator = models.TextField(
        blank=True, verbose_name="dc:creator",
        help_text="Person or organisation that created this concept<br>"
                  "If more than one list all using a semicolon ;",

    )
    contributor = models.TextField(
        blank=True, verbose_name="dc:contributor",
        help_text="Person or organisation that made contributions to this concept<br>"
                  "If more than one list all using a semicolon ;"
    )
    needs_review = models.BooleanField(
        null=True,
        help_text="Check if this concept needs to be reviewed"
    )
    date_created = models.DateTimeField(
        editable=False, default=timezone.now,
        verbose_name="dct:created"
    )
    date_modified = models.DateTimeField(
        editable=False, default=timezone.now,
        verbose_name="dct:modified"
    )
    created_by = models.ForeignKey(
        User, related_name="skos_concept_created",
        blank=True, null=True,
        on_delete=models.SET_NULL
    )

    class Meta:
        verbose_name = 'Concept'

    class MPTTMeta:
        order_insertion_by = ['pref_label']
        parent_attr = 'broader_concept'

    def get_vocabs_uri(self):
        return "{}{}".format("https://whatever", self.get_absolute_url)

    def save(self, *args, **kwargs):
        if self.notation == "":
            temp_notation = slugify(self.pref_label, allow_unicode=True)
            concepts = len(SkosConcept.objects.filter(notation=temp_notation))
            if concepts < 1:
                self.notation = temp_notation
            else:
                self.notation = "{}-{}".format(temp_notation, concepts)
        else:
            pass

        if not self.id:
            self.date_created = timezone.now()
        self.date_modified = timezone.now()
        super(SkosConcept, self).save(*args, **kwargs)

    def create_uri(self):
        mcs = self.scheme.identifier
        if self.legacy_id:
            concept_uri = f"{self.legacy_id}"
        else:
            if notation_for_uri:
                concept_uri = f"{mcs}#concept__{slugify(self.notation, allow_unicode=False)}__{self.id}"
            else:
                concept_uri = f"{mcs}#concept{self.id}"
        return concept_uri

    # change for template tag
    def creator_as_list(self):
        return self.creator.split(';')

    def contributor_as_list(self):
        return self.contributor.split(';')

    def broad_match_as_list(self):
        return self.broad_match.split(',')

    def related_as_list(self):
        return self.related.split(',')

    def narrow_match_as_list(self):
        return self.narrow_match.split(',')

    def exact_match_as_list(self):
        return self.exact_match.split(',')

    def related_match_as_list(self):
        return self.related_match.split(',')

    def close_match_as_list(self):
        return self.close_match.split(',')

    @classmethod
    def get_listview_url(self):
        return reverse('vocabs:browse_vocabs')

    @classmethod
    def get_createview_url(self):
        return reverse('vocabs:skosconcept_create')

    def get_absolute_url(self):
        return reverse('vocabs:skosconcept_detail', kwargs={'pk': self.id})

    def __str__(self):
        return self.pref_label


######################################################################
#   Classes  to store labels and notes for Concept
######################################################################


class ConceptLabel(models.Model):
    """
    A Class for Concept labels of any type.

    """
    concept = models.ForeignKey(
        SkosConcept,
        related_name="has_labels",
        verbose_name="skos:Concept",
        help_text="Which Skos:Concept current label belongs to",
        on_delete=models.CASCADE
    )
    name = models.CharField(
        max_length=500, verbose_name="label",
        help_text="Other label for this concept"
    )
    language = models.CharField(
        max_length=3, verbose_name="language",
        help_text="Language of label given above"
    )
    label_type = models.CharField(
        choices=LABEL_TYPES, verbose_name="label type",
        default='altLabel', max_length=12,
        help_text="Choose label type"
    )

    def __str__(self):
        return "{}".format(self.name)


class ConceptNote(models.Model):
    """
    A Class for SKOS documentary notes that are used
    for general documentation pusposes.

    """
    concept = models.ForeignKey(
        SkosConcept,
        related_name="has_notes",
        verbose_name="skos:Concept",
        help_text="Which Skos:Concept current documentary note belongs to",
        on_delete=models.CASCADE
    )
    name = models.TextField(
        verbose_name="documentary note",
        help_text="Provide some information about this concept"
    )
    language = models.CharField(
        max_length=3, verbose_name="language",
        help_text="Language of note given above"
    )
    note_type = models.CharField(
        choices=NOTE_TYPES, verbose_name="note type",
        default='note', max_length=15,
        help_text="Choose note type"
    )

    def __str__(self):
        return "{}".format(self.name)


class ConceptSource(models.Model):
    """
    A Class for Concept source information.

    """
    concept = models.ForeignKey(
        SkosConcept,
        related_name="has_sources",
        verbose_name="skos:Concept",
        help_text="Which Skos:Concept current source belongs to",
        on_delete=models.CASCADE
    )
    name = models.TextField(
        verbose_name="dc:source",
        help_text="Verbal description of the concept's source"
    )
    language = models.CharField(
        max_length=3, verbose_name="language",
        help_text="Language of source given above"
    )

    def __str__(self):
        return "{}".format(self.name)


def get_all_children(self, include_self=True):
    # many thanks to https://stackoverflow.com/questions/4725343
    r = []
    if include_self:
        r.append(self)
    for c in SkosConcept.objects.filter(broader_concept=self):
        _r = get_all_children(c, include_self=True)
        if 0 < len(_r):
            r.extend(_r)
    return r



from rdflib import Graph, Namespace, RDF, URIRef
from rdflib.namespace import DC, RDFS, SKOS
from .models import *
import re
import logging
from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction


logging.getLogger().setLevel(logging.INFO)

SKOS = Namespace("http://www.w3.org/2004/02/skos/core#")
DC = Namespace("http://purl.org/dc/elements/1.1/")
DCT = Namespace("http://purl.org/dc/terms/")
RDFS = Namespace("http://www.w3.org/2000/01/rdf-schema#")
OWL = Namespace("http://www.w3.org/2002/07/owl#")
VOCABS = Namespace("https://vocabs.acdh.oeaw.ac.at/create-concept-scheme/")


class SkosImporter(object):
    """
    Perform a file parsing and importing SKOS data in database
    """

    def __init__(self, file, file_format=None, language=None):
        self.file = file
        self.file_format = file_format
        self.language = language

    def _graph_read(self):
        """
        Parse a file in RDF Graph
        """
        g = Graph()
        g.bind('skos', SKOS)
        g.bind('dc', DC)
        g.bind('dct', DCT)
        g.bind('rdfs', RDFS)
        g.parse(self.file, format=self.file_format)
        return g

    def parse_triples(self):
        """
        Reads graph, finds triples about concept scheme and its concepts,
        returns a dictionary
        """
        concept_scheme = {}
        g = self._graph_read()

        def allow_properties(_property):
            """Allow DC and DCT properties"""
            properties = [URIRef('http://purl.org/dc/terms/{}'.format(_property)),
                          URIRef('http://purl.org/dc/elements/1.1/{}'.format(_property))]
            return properties

        def language_check(property_lang):
            """Check for language attributes, if they are absent set language to a specified language"""
            if property_lang:
                if str(property_lang) == "None":
                    return self.language
                else:
                    return str(property_lang)
            else:
                return self.language

        # Parsing concept scheme

        if (None, RDF.type, SKOS.ConceptScheme) in g:
            for cs in g.subjects(RDF.type, SKOS.ConceptScheme):
                concept_scheme["identifier"] = str(cs)
                titles = []
                # Set labels properties to recognize all possible labels
                for title in g.preferredLabel(
                        cs, labelProperties=((DC.title), (RDFS.label), (DCT.title), (SKOS.prefLabel))
                ):
                    temp_title = {"title": str(title[1])}
                    # If language attribute is absent populate it with a specified language
                    if str(title[1].language) == "None":
                        temp_title["lang"] = self.language
                    else:
                        temp_title["lang"] = str(title[1].language)
                    titles.append(temp_title)
                concept_scheme["title"] = titles
                concept_scheme["creator"] = ";".join(
                    [c for cp in allow_properties('creator') for c in g.objects(cs, cp)])
                concept_scheme["contributor"] = ";".join(
                    [contr for contrp in allow_properties('contributor') for contr in g.objects(cs, contrp)])
                concept_scheme["language"] = ";".join(
                    [l for lp in allow_properties('language') for l in g.objects(cs, lp)])
                concept_scheme["subject"] = ";".join(
                    [s for sp in allow_properties('subject') for s in g.objects(cs, sp)])
                concept_scheme["publisher"] = ";".join(
                    [p for pp in allow_properties('publisher') for p in g.objects(cs, pp)])
                for license in g.objects(cs, DCT.license):
                    concept_scheme["license"] = str(license)
                descriptions = []
                for descp in allow_properties('description'):
                    for d in g.objects(cs, descp):
                        temp_desc = {
                            "name": str(d),
                            "lang": language_check(d.language)
                        }
                        descriptions.append(temp_desc)
                concept_scheme["description"] = descriptions
                sources = []
                for sp in allow_properties('source'):
                    for s in g.objects(cs, sp):
                        temp_s = {
                            "name": str(s),
                            "lang": language_check(s.language)
                        }
                        sources.append(temp_s)
                concept_scheme["source"] = sources

        else:
            raise Exception("rdf:type skos:ConceptScheme is not found")

        # Parsing Collection

        if (None, RDF.type, SKOS.Collection) in g:
            collections = []
            for col in g.subjects(RDF.type, SKOS.Collection):
                collection = {"legacy_id": str(col)}
                labels = []
                for label in g.preferredLabel(col, labelProperties=(
                        (DC.title), (RDFS.label), (DCT.title), (SKOS.prefLabel))):
                    temp_col_label = {"label": str(label[1])}
                    if str(label[1].language):
                        if str(label[1].language) == "None":
                            temp_col_label["label_lang"] = self.language
                        else:
                            temp_col_label["label_lang"] = str(label[1].language)
                    else:
                        temp_col_label["label_lang"] = self.language
                    labels.append(temp_col_label)
                collection["labels"] = labels
                members = []
                for member in g.objects(col, SKOS.member):
                    members.append(str(member))
                collection["members"] = members
                # Collection notes
                col_notes = []
                col_predicates = [SKOS.note, SKOS.definition, SKOS.scopeNote, SKOS.changeNote,
                                  SKOS.editorialNote, SKOS.historyNote, SKOS.example]
                for pred in col_predicates:
                    for note in g.objects(col, pred):
                        temp_note = {
                            "name": str(note),
                            "lang": language_check(note.language),
                            "note_type": re.search('http.*#(.*)', str(pred)).group(1)
                        }
                        col_notes.append(temp_note)
                collection["note"] = col_notes
                # Collection other labels
                col_labels = []
                for pred in [SKOS.altLabel, SKOS.hiddenLabel]:
                    for other_label in g.objects(col, pred):
                        temp_other_label = {
                            "name": str(other_label),
                            "lang": language_check(other_label.language),
                            "label_type": re.search('http.*#(.*)', str(pred)).group(1)
                        }
                        col_labels.append(temp_other_label)
                collection["other_label"] = col_labels
                # Collection sources
                col_sources = []
                for col_srcp in allow_properties('source'):
                    for source in g.objects(col, col_srcp):
                        temp_col_src = {
                            "name": str(source),
                            "lang": language_check(source.language)
                        }
                        col_sources.append(temp_col_src)
                collection["source"] = col_sources
                collections.append(collection)
            concept_scheme["collections"] = collections

        else:
            pass

        # Parsing concepts triples

        if (None, RDF.type, SKOS.Concept) in g:
            concepts = []
            for c in g.subjects(RDF.type, SKOS.Concept):
                concept = {"legacy_id": str(c)}
                # Concept pref labels
                pref_labels = []
                for pref_label in g.preferredLabel(c):
                    label = {"label": str(pref_label[1])}
                    if str(pref_label[1].language) == "None":
                        label["lang"] = self.language
                    else:
                        label["lang"] = str(pref_label[1].language)
                    pref_labels.append(label)
                concept["pref_label"] = pref_labels
                for scheme in g.objects(c, SKOS.inScheme):
                    concept["scheme"] = str(scheme)
                for notation in g.objects(c, SKOS.notation):
                    concept["notation"] = str(notation)
                concept["creator"] = ";".join([c for cp in allow_properties('creator') for c in g.objects(cs, cp)])
                concept["contributor"] = ";".join(
                    [contr for contrp in allow_properties('contributor') for contr in g.objects(cs, contrp)])
                for broader_concept in g.objects(c, SKOS.broader):
                    concept["broader_concept"] = str(broader_concept)
                # Concept alt labels
                alt_labels = []
                for alt_label in g.objects(c, SKOS.altLabel):
                    label = {
                        "label": str(alt_label),
                        "lang": language_check(alt_label.language)
                    }
                    alt_labels.append(label)
                concept["alt_label"] = alt_labels
                # Concept hidden labels
                hidden_labels = []
                for hidden_label in g.objects(c, SKOS.hiddenLabel):
                    label = {
                        "label": str(hidden_label),
                        "lang": language_check(hidden_label.language)
                    }
                    hidden_labels.append(label)
                concept["hidden_label"] = hidden_labels
                # Concept sources
                sources = []
                for srcp in allow_properties('source'):
                    for source in g.objects(c, srcp):
                        temp_source = {
                            "name": str(source),
                            "lang": language_check(source.language)
                        }
                        sources.append(temp_source)
                concept["source"] = sources
                # Concept documentary notes
                notes = []
                predicates = [SKOS.note, SKOS.definition, SKOS.scopeNote, SKOS.changeNote,
                              SKOS.editorialNote, SKOS.historyNote, SKOS.example]
                for pred in predicates:
                    for note in g.objects(c, pred):
                        temp_note = {
                            "name": str(note),
                            "lang": language_check(note.language),
                            "note_type": re.search('http.*#(.*)', str(pred)).group(1)
                        }
                        notes.append(temp_note)
                concept["note"] = notes
                # Add concept to a list
                concepts.append(concept)
            concept_scheme["has_concepts"] = concepts
        else:
            pass
            logging.info("Graph doesn't have concepts")
        return concept_scheme

    def upload_data(self, user):
        """
        Creates and saves concept scheme and its concepts in a database
        """
        concept_scheme = self.parse_triples()
        concept_scheme_uri = concept_scheme.get("identifier")
        concept_scheme_creator = concept_scheme.get("creator", "")
        concept_scheme_contributor = concept_scheme.get("contributor", "")
        concept_scheme_language = concept_scheme.get("language", "")
        concept_scheme_subject = concept_scheme.get("subject", "")
        concept_scheme_publisher = concept_scheme.get("publisher", "")
        concept_scheme_license = concept_scheme.get("license", "")
        concept_scheme_description = concept_scheme.get("description")
        concept_scheme_source = concept_scheme.get("source")
        concept_scheme_has_concepts = concept_scheme.get("has_concepts")
        concept_scheme_has_collections = concept_scheme.get("collections")
        main_title = {}
        other_titles = []
        for title in concept_scheme.get("title"):
            if title.get("lang") == self.language:
                main_title["label"] = title.get("title")
                main_title["lang"] = title.get("lang")
            else:
                other_title = {
                    "label": title.get("title", "Empty title"),
                    "lang": title.get("lang", self.language)
                }
                other_titles.append(other_title)
        concept_scheme_title = main_title.get("label", "No title in specified language")
        concept_scheme_title_lang = main_title.get("lang", self.language)
        concept_scheme = SkosConceptScheme.objects.create(
            identifier=concept_scheme_uri,
            title=concept_scheme_title, title_lang=concept_scheme_title_lang,
            creator=concept_scheme_creator, contributor=concept_scheme_contributor,
            language=concept_scheme_language, subject=concept_scheme_subject,
            publisher=concept_scheme_publisher, license=concept_scheme_license,
            created_by=User.objects.get(username=user)
        )
        if len(other_titles) > 0:
            for other in other_titles:
                ConceptSchemeTitle.objects.create(
                    concept_scheme=concept_scheme, name=other.get("label"),
                    language=other.get("lang")
                )
        else:
            pass

        if concept_scheme_description:
            for desc in concept_scheme_description:
                ConceptSchemeDescription.objects.create(
                    concept_scheme=concept_scheme, name=desc.get("name"),
                    language=desc.get("lang")
                )
        else:
            pass
        if concept_scheme_source:
            for source in concept_scheme_source:
                ConceptSchemeSource.objects.create(
                    concept_scheme=concept_scheme, name=source.get("name"),
                    language=source.get("lang")
                )
        else:
            pass

        with transaction.atomic():
            if concept_scheme_has_collections:

                for col in concept_scheme_has_collections:
                    col_notes = col.get("note", "")
                    col_alt_hid_labels = col.get("other_label", "")
                    col_sources = col.get("source", "")
                    col_main_label = {}
                    col_other_labels = []
                    for label in col.get("labels"):
                        if label.get("label_lang") == self.language:
                            col_main_label["label"] = label.get("label")
                            col_main_label["lang"] = label.get("label_lang")
                        else:
                            other_label = {
                                "label": label.get("label", "other label"),
                                "lang": label.get("label_lang", self.language)
                            }
                            col_other_labels.append(other_label)
                    new_collection = SkosCollection.objects.create(scheme=concept_scheme,
                                                                   name=col_main_label.get("label",
                                                                                           "no label in specified "
                                                                                           "language"),
                                                                   legacy_id=col.get("legacy_id"),
                                                                   label_lang=col_main_label.get("lang", self.language),
                                                                   created_by=User.objects.get(username=user))
                    if len(col_other_labels) > 0:
                        for other in col_other_labels:
                            CollectionLabel.objects.create(
                                collection=new_collection, name=other.get("label"),
                                language=other.get("lang"), label_type="prefLabel"
                            )
                    if col_notes:
                        for cn in col_notes:
                            CollectionNote.objects.create(
                                collection=new_collection, name=cn.get("name"),
                                language=cn.get("lang"), note_type=cn.get("note_type")
                            )
                    if col_alt_hid_labels:
                        for cahl in col_alt_hid_labels:
                            CollectionLabel.objects.create(
                                collection=new_collection, name=cahl.get("name"),
                                language=cahl.get("lang"), label_type=cahl.get("label_type")
                            )
                    if col_sources:
                        for csrc in col_sources:
                            CollectionSource.objects.create(
                                collection=new_collection, name=csrc.get("name"),
                                language=csrc.get("lang")
                            )

            else:
                pass

        if concept_scheme_has_concepts:

            for concept in concept_scheme_has_concepts:
                concept_legacy_id = concept.get("legacy_id")
                concept_inscheme = concept.get("scheme")
                concept_notation = concept.get("notation", "")
                concept_creator = concept.get("creator", "")
                concept_contributor = concept.get("contributor", "")
                concept_alt_labels = concept.get("alt_label")
                concept_hidden_labels = concept.get("hidden_label")
                concept_notes = concept.get("note")
                concept_sources = concept.get("source")
                main_pref_label = {}
                other_pref_labels = []
                for pref_label in concept.get("pref_label"):
                    if pref_label.get("lang") == self.language:
                        main_pref_label["label"] = pref_label.get("label")
                        main_pref_label["lang"] = pref_label.get("lang")
                    else:
                        other_pref_label = {
                            "label": pref_label.get("label"),
                            "lang": pref_label.get("lang", self.language)
                        }
                        other_pref_labels.append(other_pref_label)
                concept_pref_label = main_pref_label.get("label", "no label in this language")
                concept_pref_label_lang = main_pref_label.get("lang", self.language)
                new_concept = SkosConcept.objects.create(
                    legacy_id=concept_legacy_id,
                    scheme=SkosConceptScheme.objects.get(id=concept_scheme.id),
                    pref_label=concept_pref_label, pref_label_lang=concept_pref_label_lang,
                    notation=concept_notation, creator=concept_creator,
                    contributor=concept_contributor, created_by=User.objects.get(username=user)
                )
                new_concept.save()

                # concept to collections
                if concept_scheme_has_collections:
                    collections = []
                    for col in concept_scheme_has_collections:
                        if concept_legacy_id in col.get("members"):
                            collections.append(col.get("legacy_id"))
                        else:
                            pass
                    if len(collections) > 0:
                        member_of_collections = SkosCollection.objects.filter(legacy_id__in=collections)
                        new_concept.collection.set(member_of_collections)
                else:
                    pass

                if len(other_pref_labels) > 0:
                    for other in other_pref_labels:
                        ConceptLabel.objects.create(
                            concept=new_concept, name=other.get("label"),
                            language=other.get("lang"), label_type="prefLabel"
                        )
                else:
                    pass
                if len(concept_alt_labels) > 0:
                    for alt in concept_alt_labels:
                        ConceptLabel.objects.create(
                            concept=new_concept, name=alt.get("label"),
                            language=alt.get("lang"), label_type="altLabel"
                        )
                else:
                    pass
                if len(concept_hidden_labels) > 0:
                    for hid in concept_hidden_labels:
                        ConceptLabel.objects.create(
                            concept=new_concept, name=hid.get("label"),
                            language=hid.get("lang"), label_type="hiddenLabel"
                        )
                else:
                    pass
                if len(concept_notes) > 0:
                    for n in concept_notes:
                        ConceptNote.objects.create(
                            concept=new_concept, name=n.get("name"),
                            language=n.get("lang"), note_type=n.get("note_type")
                        )
                else:
                    pass
                if len(concept_sources) > 0:
                    for s in concept_sources:
                        ConceptSource.objects.create(
                            concept=new_concept, name=s.get("name"),
                            language=s.get("lang")
                        )
                else:
                    pass
            # add relationships
            for concept in concept_scheme_has_concepts:
                if concept.get("broader_concept") is not None:
                    local_concepts = SkosConcept.objects.filter(scheme=concept_scheme.id)
                    try:
                        local_concepts.filter(
                            legacy_id=concept.get("legacy_id")).update(
                            broader_concept=local_concepts.get(legacy_id=concept.get("broader_concept"))
                        )
                    except ObjectDoesNotExist as e:
                        pass
                        logging.info(e)
                else:
                    pass
            return SkosConcept.objects.rebuild()
        else:
            pass
        return



from dal import autocomplete
from django import forms
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit, Layout, Fieldset, Div, HTML, ButtonHolder
from crispy_forms.bootstrap import *
from .models import *
from django.forms.models import inlineformset_factory
from .custom_layout_object import *
from mptt.forms import TreeNodeChoiceField
from django.forms import BaseInlineFormSet
import re
from .endpoints import *


class GenericFilterFormHelper(FormHelper):
    def __init__(self, *args, **kwargs):
        super(GenericFilterFormHelper, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.form_class = 'genericFilterForm'
        self.form_method = 'GET'
        self.add_input(Submit('Filter', 'search'))


class UploadFileForm(forms.Form):
    file = forms.FileField()
    language = forms.CharField(
        max_length=3, required=True,
        help_text="Specify the main language of your vocabulary (in format ISO 639-1 or ISO 639-3)",
        widget=forms.TextInput(attrs={'placeholder': 'e.g. en'})
    )

    def __init__(self, *args, **kwargs):
        super(UploadFileForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-3'
        self.helper.field_class = 'col-md-9'
        self.helper.add_input(Submit('submit', 'upload', css_id='uploadFile'), )


def custom_name_errors(field_name):
    name_errors = {'required': '{} is required when language provided'.format(field_name)}
    name_errors['invalid']: 'Enter a valid value'
    return name_errors


def custom_lang_errors(field_name):
    lang_errors = {'required': 'Language is required when {} provided'.format(field_name.lower())}
    lang_errors['invalid']: 'Enter a valid value'
    return lang_errors


######################################################################
#   Classes  to store titles and descriptions for ConceptScheme
######################################################################


# We also want to verify that all formsets have both fields a name and a language filled out.
# We could simply set the fields as required on the form itself,
# however this will prevent our users from submitting empty forms,
# which is not the behaviour we’re looking for here. From a usability perspective, it would be better
# to simply ignore forms that are completely empty, raising errors only if a form is partially incomplete.

class CustomInlineFormSet(BaseInlineFormSet):
    def clean(self):
        super(CustomInlineFormSet, self).clean()
        if any(self.errors):
            return
        for form in self.forms:
            if form.cleaned_data:
                name = form.cleaned_data['name']
                language = form.cleaned_data['language']
                # Check that formset has both fields - a name and a language - filled out
                if name and not language:
                    raise forms.ValidationError(
                        'All names must have a lanaguge.'
                    )
                elif language and not name:
                    raise forms.ValidationError(
                        'All languages must have a name.'
                    )


class ConceptSchemeTitleForm(forms.ModelForm):
    name = forms.CharField(
        label=ConceptSchemeTitle._meta.get_field('name').verbose_name,
        help_text=ConceptSchemeTitle._meta.get_field('name').help_text,
        error_messages=custom_name_errors(
            field_name=ConceptSchemeTitle._meta.get_field('name').verbose_name
        )
    )
    language = forms.CharField(
        label=ConceptSchemeTitle._meta.get_field('language').verbose_name,
        help_text=ConceptSchemeTitle._meta.get_field('language').help_text,
        error_messages=custom_lang_errors(
            field_name=ConceptSchemeTitle._meta.get_field('name').verbose_name
        )
    )

    def __init__(self, *args, **kwargs):
        super(ConceptSchemeTitleForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2 create-label'
        self.helper.field_class = 'col-md-10'

    class Meta:
        model = ConceptSchemeTitle
        exclude = ()


ConceptSchemeTitleFormSet = inlineformset_factory(
    SkosConceptScheme, ConceptSchemeTitle,
    form=ConceptSchemeTitleForm,
    fields=['name', 'language'], extra=1, can_delete=True
)


class ConceptSchemeDescriptionForm(forms.ModelForm):
    name = forms.CharField(
        label=ConceptSchemeDescription._meta.get_field('name').verbose_name,
        help_text=ConceptSchemeDescription._meta.get_field('name').help_text,
        error_messages=custom_name_errors(
            field_name=ConceptSchemeDescription._meta.get_field('name').verbose_name
        ),
        widget=forms.Textarea
    )
    language = forms.CharField(
        label=ConceptSchemeDescription._meta.get_field('language').verbose_name,
        help_text=ConceptSchemeDescription._meta.get_field('language').help_text,
        error_messages=custom_lang_errors(
            field_name=ConceptSchemeDescription._meta.get_field('name').verbose_name
        )
    )

    def __init__(self, *args, **kwargs):
        super(ConceptSchemeDescriptionForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2 create-label'
        self.helper.field_class = 'col-md-10'

    class Meta:
        model = ConceptSchemeDescription
        exclude = ()


ConceptSchemeDescriptionFormSet = inlineformset_factory(
    SkosConceptScheme, ConceptSchemeDescription,
    form=ConceptSchemeDescriptionForm,
    fields=['name', 'language'], extra=1, can_delete=True
)


class ConceptSchemeSourceForm(forms.ModelForm):
    name = forms.CharField(
        label=ConceptSchemeSource._meta.get_field('name').verbose_name,
        help_text=ConceptSchemeSource._meta.get_field('name').help_text,
        error_messages=custom_name_errors(
            field_name=ConceptSchemeSource._meta.get_field('name').verbose_name
        ),
        widget=forms.Textarea
    )
    language = forms.CharField(
        label=ConceptSchemeSource._meta.get_field('language').verbose_name,
        help_text=ConceptSchemeSource._meta.get_field('language').help_text,
        error_messages=custom_lang_errors(
            field_name=ConceptSchemeSource._meta.get_field('name').verbose_name
        )
    )

    def __init__(self, *args, **kwargs):
        super(ConceptSchemeSourceForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2 create-label'
        self.helper.field_class = 'col-md-10'

    class Meta:
        model = ConceptSchemeSource
        exclude = ()


ConceptSchemeSourceFormSet = inlineformset_factory(
    SkosConceptScheme, ConceptSchemeSource, form=ConceptSchemeSourceForm,
    fields=['name', 'language'],
    extra=1, can_delete=True
)


######################################################################
#
# SkosConceptScheme
#
######################################################################


class SkosConceptSchemeForm(forms.ModelForm):
    class Meta:
        model = SkosConceptScheme
        exclude = ['created_by', ]
        widgets = {
            'curator': autocomplete.ModelSelect2Multiple(
                url='vocabs-ac:user-autocomplete'),
        }

    def __init__(self, *args, **kwargs):
        super(SkosConceptSchemeForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2 create-label'
        self.helper.field_class = 'col-md-10'
        self.helper.layout = Layout(
            Div(
                Field('title'),
                Field('title_lang'),
                Fieldset('Add titles in other languages',
                         Formset('titles'), css_class="formset-div"),
                Fieldset('Add description',
                         Formset('descriptions'), css_class="formset-div"),
                Field('identifier', placeholder="https://example.org/vocabulary-unique-title"),
                Field('language'),
                Field('creator'),
                Field('contributor'),
                Field('publisher'),
                Field('license'),
                Field('owner'),
                Field('subject'),
                Field('relation'),
                Field('coverage'),
                Fieldset('Add source information',
                         Formset('sources'), css_class="formset-div"),
                Field('version'),
                Field('legacy_id'),
                Field('date_issued', placeholder="YYYY-MM-DD"),
                Field('curator'),
                HTML("<br>"),
                ButtonHolder(Submit('submit', 'save')),
            )
        )


class SkosConceptSchemeFormHelper(FormHelper):
    def __init__(self, *args, **kwargs):
        super(SkosConceptSchemeFormHelper, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.form_class = 'genericFilterForm'
        self.form_method = 'GET'
        self.helper.form_tag = False
        self.add_input(Submit('Filter', 'Search'))
        self.layout = Layout(
            Fieldset(
                '',
                'title',
                'creator',
                css_id="basic_search_fields"
            ),
        )


######################################################################
#   Classes  to store labels and notes for Collection
######################################################################

class CollectionLabelForm(forms.ModelForm):
    name = forms.CharField(
        label=CollectionLabel._meta.get_field('name').verbose_name,
        help_text=CollectionLabel._meta.get_field('name').help_text,
        error_messages=custom_name_errors(
            field_name=CollectionLabel._meta.get_field('name').verbose_name
        )
    )
    language = forms.CharField(
        label=CollectionLabel._meta.get_field('language').verbose_name,
        help_text=CollectionLabel._meta.get_field('language').help_text,
        error_messages=custom_lang_errors(
            field_name=CollectionLabel._meta.get_field('name').verbose_name
        )
    )

    def __init__(self, *args, **kwargs):
        super(CollectionLabelForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2 create-label'
        self.helper.field_class = 'col-md-10'

    class Meta:
        model = CollectionLabel
        exclude = ()


CollectionLabelFormSet = inlineformset_factory(
    SkosCollection, CollectionLabel, form=CollectionLabelForm,
    fields=['name', 'label_type', 'language'],
    extra=1, can_delete=True
)


class CollectionNoteForm(forms.ModelForm):
    name = forms.CharField(
        label=CollectionNote._meta.get_field('name').verbose_name,
        help_text=CollectionNote._meta.get_field('name').help_text,
        error_messages=custom_name_errors(
            field_name=CollectionNote._meta.get_field('name').verbose_name
        ),
        widget=forms.Textarea
    )
    language = forms.CharField(
        label=CollectionNote._meta.get_field('language').verbose_name,
        help_text=CollectionNote._meta.get_field('language').help_text,
        error_messages=custom_lang_errors(
            field_name=CollectionNote._meta.get_field('name').verbose_name
        )
    )

    def __init__(self, *args, **kwargs):
        super(CollectionNoteForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2 create-label'
        self.helper.field_class = 'col-md-10'

    class Meta:
        model = CollectionNote
        exclude = ()


CollectionNoteFormSet = inlineformset_factory(
    SkosCollection, CollectionNote, form=CollectionNoteForm,
    fields=['name', 'note_type', 'language'],
    extra=1, can_delete=True
)


class CollectionSourceForm(forms.ModelForm):
    name = forms.CharField(
        label=CollectionSource._meta.get_field('name').verbose_name,
        help_text=CollectionSource._meta.get_field('name').help_text,
        error_messages=custom_name_errors(
            field_name=CollectionSource._meta.get_field('name').verbose_name
        ),
        widget=forms.Textarea
    )
    language = forms.CharField(
        label=CollectionSource._meta.get_field('language').verbose_name,
        help_text=CollectionSource._meta.get_field('language').help_text,
        error_messages=custom_lang_errors(
            field_name=CollectionSource._meta.get_field('name').verbose_name
        )
    )

    def __init__(self, *args, **kwargs):
        super(CollectionSourceForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2 create-label'
        self.helper.field_class = 'col-md-10'

    class Meta:
        model = CollectionSource
        exclude = ()


CollectionSourceFormSet = inlineformset_factory(
    SkosCollection, CollectionSource, form=CollectionSourceForm,
    fields=['name', 'language'],
    extra=1, can_delete=True
)


######################################################################
#
# SkosCollection
#
######################################################################

class SkosCollectionForm(forms.ModelForm):
    class Meta:
        model = SkosCollection
        exclude = ['created_by', ]
        widgets = {
            'scheme': autocomplete.ModelSelect2(
                url='vocabs-ac:skosconceptscheme-autocomplete'),
        }

    def __init__(self, *args, **kwargs):
        super(SkosCollectionForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2 create-label'
        self.helper.field_class = 'col-md-10'
        self.helper.layout = Layout(
            Div(
                Field('name'),
                Field('label_lang'),
                Fieldset('Add other labels or labels in other languages',
                         Formset('labels'), css_class="formset-div"),
                Field('scheme'),
                Field('creator'),
                Field('contributor'),
                Field('legacy_id'),
                Fieldset('Add documentary note',
                         Formset('notes'), css_class="formset-div"),
                Fieldset('Add source information',
                         Formset('sources'), css_class="formset-div"),
                HTML("<br>"),
                ButtonHolder(Submit('submit', 'save')),
            )
        )


class SkosCollectionFormHelper(FormHelper):

    def __init__(self, *args, **kwargs):
        super(SkosCollectionFormHelper, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.form_class = 'genericFilterForm'
        self.form_method = 'GET'
        self.helper.form_tag = False
        self.add_input(Submit('Filter', 'Search'))
        self.layout = Layout(
            Fieldset(
                '',
                'name',
                'creator',
                'scheme',
                'has_members__pref_label',
                css_id="basic_search_fields"
            ),
        )


######################################################################
#   Classes  to store labels, notes and sources for Concept
######################################################################

class ConceptLabelForm(forms.ModelForm):
    name = forms.CharField(
        label=ConceptLabel._meta.get_field('name').verbose_name,
        help_text=ConceptLabel._meta.get_field('name').help_text,
        error_messages=custom_name_errors(
            field_name=ConceptLabel._meta.get_field('name').verbose_name
        )
    )
    language = forms.CharField(
        label=ConceptLabel._meta.get_field('language').verbose_name,
        help_text=ConceptLabel._meta.get_field('language').help_text,
        error_messages=custom_lang_errors(
            field_name=ConceptLabel._meta.get_field('name').verbose_name
        )
    )

    def __init__(self, *args, **kwargs):
        super(ConceptLabelForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2 create-label'
        self.helper.field_class = 'col-md-10'

    class Meta:
        model = ConceptLabel
        exclude = ()


ConceptLabelFormSet = inlineformset_factory(
    SkosConcept, ConceptLabel, form=ConceptLabelForm,
    fields=['name', 'label_type', 'language'],
    extra=1, can_delete=True
)


class ConceptNoteForm(forms.ModelForm):
    name = forms.CharField(
        label=ConceptNote._meta.get_field('name').verbose_name,
        help_text=ConceptNote._meta.get_field('name').help_text,
        error_messages=custom_name_errors(
            field_name=ConceptNote._meta.get_field('name').verbose_name
        ),
        widget=forms.Textarea
    )
    language = forms.CharField(
        label=ConceptNote._meta.get_field('language').verbose_name,
        help_text=ConceptNote._meta.get_field('language').help_text,
        error_messages=custom_lang_errors(
            field_name=ConceptNote._meta.get_field('name').verbose_name
        )
    )

    def __init__(self, *args, **kwargs):
        super(ConceptNoteForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2 create-label'
        self.helper.field_class = 'col-md-10'

    class Meta:
        model = ConceptNote
        exclude = ()


ConceptNoteFormSet = inlineformset_factory(
    SkosConcept, ConceptNote, form=ConceptNoteForm,
    fields=['name', 'note_type', 'language'],
    extra=1, can_delete=True
)


class ConceptSourceForm(forms.ModelForm):
    name = forms.CharField(
        label=ConceptSource._meta.get_field('name').verbose_name,
        help_text=ConceptSource._meta.get_field('name').help_text,
        error_messages=custom_name_errors(
            field_name=ConceptSource._meta.get_field('name').verbose_name
        ),
        widget=forms.Textarea
    )
    language = forms.CharField(
        label=ConceptSource._meta.get_field('language').verbose_name,
        help_text=ConceptSource._meta.get_field('language').help_text,
        error_messages=custom_lang_errors(
            field_name=ConceptSource._meta.get_field('name').verbose_name
        )
    )

    def __init__(self, *args, **kwargs):
        super(ConceptSourceForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2 create-label'
        self.helper.field_class = 'col-md-10'

    class Meta:
        model = ConceptSource
        exclude = ()


ConceptSourceFormSet = inlineformset_factory(
    SkosConcept, ConceptSource, form=ConceptSourceForm,
    fields=['name', 'language'],
    extra=1, can_delete=True
)


######################################################################
#
# SkosConcept
#
######################################################################

class AutocompleteCharField(forms.CharField):
    widget = autocomplete.TagSelect2(
        url='vocabs-ac:external-link-ac',
        forward=['endpoint'],
        attrs={
            'data-placeholder': 'Type at least 3 characters to get autocomplete suggestions ...',
            'data-minimum-input-length': 3,
        },
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.required = False

    def to_python(self, value):
        """Normalize data to keep only URIs."""
        clean_value = ",".join(re.findall("(?P<url>https?://[^\s\,]+)", value))
        return clean_value


class SkosConceptForm(forms.ModelForm):
    broader_concept = TreeNodeChoiceField(
        queryset=SkosConcept.objects.all(),
        widget=autocomplete.ModelSelect2(
            url='vocabs-ac:skosconcept-autocomplete',
            forward=['scheme']
        ),
        help_text=SkosConcept._meta.get_field('broader_concept').help_text,
        required=False,
        label=SkosConcept._meta.get_field('broader_concept').verbose_name,
    )
    collection = forms.ModelMultipleChoiceField(
        queryset=SkosCollection.objects.all(),
        widget=autocomplete.ModelSelect2Multiple(
            url='vocabs-ac:skoscollection-autocomplete',
            forward=['scheme']
        ),
        help_text=SkosConcept._meta.get_field('collection').help_text,
        required=False,
        label=SkosConcept._meta.get_field('collection').verbose_name,
    )
    endpoint = forms.ChoiceField(
        choices=ENDPOINT_CHOICES, required=False,
        help_text="Select a service to create links to external resources<br>\
        You can also type a matching concept URI in the fields below if it is not provided by current endpoints<br>\
        In that case please note that an external concept's URI should follow the format 'http{s}://example.org/...'"
    )
    related = AutocompleteCharField(
        label=SkosConcept._meta.get_field('related').verbose_name,
        help_text=SkosConcept._meta.get_field('related').help_text
    )
    broad_match = AutocompleteCharField(
        label=SkosConcept._meta.get_field('broad_match').verbose_name,
        help_text=SkosConcept._meta.get_field('broad_match').help_text
    )
    narrow_match = AutocompleteCharField(
        label=SkosConcept._meta.get_field('narrow_match').verbose_name,
        help_text=SkosConcept._meta.get_field('narrow_match').help_text
    )
    exact_match = AutocompleteCharField(
        label=SkosConcept._meta.get_field('exact_match').verbose_name,
        help_text=SkosConcept._meta.get_field('exact_match').help_text
    )
    related_match = AutocompleteCharField(
        label=SkosConcept._meta.get_field('related_match').verbose_name,
        help_text=SkosConcept._meta.get_field('related_match').help_text
    )
    close_match = AutocompleteCharField(
        label=SkosConcept._meta.get_field('close_match').verbose_name,
        help_text=SkosConcept._meta.get_field('close_match').help_text
    )
    needs_review = forms.BooleanField(
        widget=forms.CheckboxInput, required=False,
        help_text=SkosConcept._meta.get_field('needs_review').help_text
    )

    class Meta:
        model = SkosConcept
        exclude = ['created_by', ]
        widgets = {
            'scheme': autocomplete.ModelSelect2(
                url='vocabs-ac:skosconceptscheme-autocomplete'),
        }

    def __init__(self, *args, **kwargs):
        super(SkosConceptForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = True
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-2 create-label'
        self.helper.field_class = 'col-md-10'
        self.helper.layout = Layout(
            Div(
                Field('pref_label'),
                Field('pref_label_lang'),
                Fieldset('Add other labels',
                         Formset('labels'), css_class="formset-div"),
                Field('scheme'),
                # Field('top_concept'),
                Field('collection'),
                Field('broader_concept'),
                Fieldset('Add documentary note',
                         Formset('notes'), css_class="formset-div"),
                Field('creator'),
                Field('contributor'),
                Field('notation'),
                Field('legacy_id'),
                Fieldset('Add source information',
                         Formset('sources'), css_class="formset-div"),
                Fieldset('Add semantic relationships',
                         Field('endpoint'),
                         Field('related'),
                         Field('broad_match'),
                         Field('narrow_match'),
                         Field('exact_match'),
                         Field('related_match'),
                         Field('close_match'),
                         ),
                Fieldset('Quality check',
                         Field('needs_review'),
                         ),
                HTML("<br>"),
                ButtonHolder(Submit('submit', 'save')),
            )
        )
        self.helper.render_required_fields = True


class SkosConceptFormHelper(FormHelper):
    def __init__(self, *args, **kwargs):
        super(SkosConceptFormHelper, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.form_class = 'genericFilterForm'
        self.form_method = 'GET'
        self.helper.form_tag = False
        self.add_input(Submit('Filter', 'Search'))
        self.layout = Layout(
            Fieldset(
                '',
                'pref_label',
                'scheme',
                'collection',
                'broader_concept',
                css_id="basic_search_fields"
            ),
        )



import django_filters
from dal import autocomplete
from .models import SkosConcept, SkosConceptScheme, SkosCollection


django_filters.filters.LOOKUP_TYPES = [
    ('', '---------'),
    ('exact', 'Is equal to'),
    ('iexact', 'Is equal to (case insensitive)'),
    ('not_exact', 'Is not equal to'),
    ('lt', 'Lesser than/before'),
    ('gt', 'Greater than/after'),
    ('gte', 'Greater than or equal to'),
    ('lte', 'Lesser than or equal to'),
    ('startswith', 'Starts with'),
    ('endswith', 'Ends with'),
    ('contains', 'Contains'),
    ('icontains', 'Contains (case insensitive)'),
    ('not_contains', 'Does not contain'),
]


class SkosConceptListFilter(django_filters.FilterSet):
    pref_label = django_filters.ModelMultipleChoiceFilter(
        widget=autocomplete.Select2Multiple(
            url='vocabs-ac:skosconcept-autocomplete',
        ),
        queryset=SkosConcept.objects.all(),
        lookup_expr='icontains',
        label='Skos:prefLabel',
        help_text=False,
    )
    scheme = django_filters.ModelChoiceFilter(
        queryset=SkosConceptScheme.objects.all(),
        widget=autocomplete.ModelSelect2(
            url='vocabs-ac:skosconceptscheme-autocomplete',
            attrs={
                'data-placeholder': 'Start typing ...',
            }
        ),
        help_text=False,
    )
    collection = django_filters.ModelChoiceFilter(
        queryset=SkosCollection.objects.all(),
        widget=autocomplete.ModelSelect2(
            url='vocabs-ac:skoscollection-autocomplete',
            attrs={
                'data-placeholder': 'Start typing ...',
            },
            forward=['scheme']
        ),
        help_text=False,
    )
    broader_concept = django_filters.ModelChoiceFilter(
        widget=autocomplete.ModelSelect2(
            url='vocabs-ac:skosconcept-autocomplete',
            forward=['scheme']
        ),
        queryset=SkosConcept.objects.all(),
        help_text='Returns all narrower concepts of a selected concept (descendants)',
        method='filter_get_descendants'
    )

    class Meta:
        model = SkosConcept
        fields = '__all__'

    def filter_get_descendants(self, qs, name, value):
        if value:
            qs = SkosConcept.objects.get_queryset_descendants(
                SkosConcept.objects.filter(pref_label=value),
                include_self=False)
            return qs
        return qs


class SkosConceptSchemeListFilter(django_filters.FilterSet):
    title = django_filters.CharFilter(
        lookup_expr='icontains',
        help_text=SkosConceptScheme._meta.get_field('title').help_text,
        label=SkosConceptScheme._meta.get_field('title').verbose_name
    )
    creator = django_filters.CharFilter(
        lookup_expr='icontains',
        help_text=False,
        label=SkosConceptScheme._meta.get_field('creator').verbose_name
    )

    class Meta:
        model = SkosConceptScheme
        fields = '__all__'


class SkosCollectionListFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(
        lookup_expr='icontains',
        label=SkosCollection._meta.get_field('name').verbose_name
    )
    creator = django_filters.CharFilter(
        lookup_expr='icontains',
        label=SkosCollection._meta.get_field('creator').verbose_name
    )
    scheme = django_filters.ModelChoiceFilter(
        queryset=SkosConceptScheme.objects.all(),
        widget=autocomplete.ModelSelect2(
            url='vocabs-ac:skosconceptscheme-autocomplete',
            attrs={
                'data-placeholder': 'Start typing ...',
            }
        ),
        help_text=False,
    )
    has_members__pref_label = django_filters.ModelMultipleChoiceFilter(
        widget=autocomplete.Select2Multiple(url='vocabs-ac:skosconcept-autocomplete'),
        queryset=SkosConcept.objects.all(),
        lookup_expr='icontains',
        help_text=False,
    )

    class Meta:
        model = SkosCollection
        fields = '__all__'



USER = {
    "username": "testuser",
    "password": "12345"
}


def concept_scheme(user):
    return {
        "title": "Test Concept Scheme",
        "title_lang": "en",
        "created_by": user
    }


def collection(concept_scheme, user):
    return {
        "name": "Test Collection",
        "label_lang": "en",
        "scheme": concept_scheme,
        "created_by": user
    }


def concept(concept_scheme, label, user, broader=None):
    return {
        "pref_label": label,
        "pref_label_lang": "en",
        "scheme": concept_scheme,
        "broader_concept": broader,
        "created_by": user
    }



from django.conf.urls import url
from . import dal_views
from .models import SkosConcept, SkosConceptScheme, SkosCollection
from django.contrib.auth.models import User


app_name = 'vocabs'

urlpatterns = [
    url(
        r'^external-link-ac/$', dal_views.ExternalLinkAC.as_view(),
        name='external-link-ac',
    ),
    url(
        r'^skosconceptscheme-autocomplete/$', dal_views.SkosConceptSchemeAC.as_view(
            model=SkosConceptScheme),
        name='skosconceptscheme-autocomplete',
    ),
    url(
        r'^skoscollection-autocomplete/$', dal_views.SkosCollectionAC.as_view(
            model=SkosCollection),
        name='skoscollection-autocomplete',
    ),
    url(
        r'^skosconcept-autocomplete/$', dal_views.SkosConceptAC.as_view(
            model=SkosConcept),
        name='skosconcept-autocomplete',
    ),
    url(
        r'^skosconcept-extmatch-autocomplete/$', dal_views.SkosConceptExternalMatchAC.as_view(
            model=SkosConcept),
        name='skosconcept-extmatch-autocomplete',
    ),
    url(
        r'^user-autocomplete/$', dal_views.UserAC.as_view(
            model=User),
        name='user-autocomplete',
    ),
]



import rdflib
from rdflib import Literal, Namespace, RDF, URIRef, XSD
from rdflib.namespace import DC, RDFS, SKOS


SKOS = Namespace("http://www.w3.org/2004/02/skos/core#")
DC = Namespace("http://purl.org/dc/elements/1.1/")
DCT = Namespace("http://purl.org/dc/terms/")
RDFS = Namespace("http://www.w3.org/2000/01/rdf-schema#")
OWL = Namespace("http://www.w3.org/2002/07/owl#")
VOCABS = Namespace("https://vocabs.acdh.oeaw.ac.at/create-concept-scheme/")


def graph_construct_qs(results):
    g = rdflib.Graph()
    g.bind('skos', SKOS)
    g.bind('dc', DC)
    g.bind('dct', DCT)
    g.bind('rdfs', RDFS)
    g.bind('owl', OWL)
    for obj in results:
        # Creating Main Concept Scheme
        if obj.scheme:
            main_concept_scheme = URIRef(obj.scheme.identifier)
            g.add((main_concept_scheme, RDF.type, SKOS.ConceptScheme))
            # Concept Scheme properties
            if obj.scheme.title:
                g.add((main_concept_scheme, DC.title, Literal(obj.scheme.title, lang=obj.scheme.title_lang)))
                g.add((main_concept_scheme, RDFS.label, Literal(obj.scheme.title, lang=obj.scheme.title_lang)))
            if obj.scheme.has_titles.all():
                for title in obj.scheme.has_titles.all():
                    g.add((main_concept_scheme, DC.title, Literal(title.name, lang=title.language)))
            if obj.scheme.has_descriptions.all():
                for desc in obj.scheme.has_descriptions.all():
                    g.add((main_concept_scheme, DC.description, Literal(desc.name, lang=desc.language)))
            if obj.scheme.has_sources.all():
                for source in obj.scheme.has_sources.all():
                    g.add((main_concept_scheme, DC.source, Literal(source.name, lang=source.language)))
            # accessing lists with ; in TextField
            if obj.scheme.creator:
                for i in obj.scheme.creator.split(';'):
                    g.add((main_concept_scheme, DC.creator, Literal(i.strip())))
            if obj.scheme.contributor:
                for i in obj.scheme.contributor.split(';'):
                    g.add((main_concept_scheme, DC.contributor, Literal(i.strip())))
            if obj.scheme.language:
                for i in obj.scheme.language.split(';'):
                    g.add((main_concept_scheme, DC.language, Literal(i.strip())))
            if obj.scheme.subject:
                for i in obj.scheme.subject.split(';'):
                    g.add((main_concept_scheme, DC.subject, Literal(i.strip())))
            if obj.scheme.coverage:
                for i in obj.scheme.coverage.split(';'):
                    g.add((main_concept_scheme, DC.coverage, Literal(i.strip())))
            # the rest of the properties
            if obj.scheme.license:
                g.add((main_concept_scheme, DCT.license, Literal(obj.scheme.license)))
            if obj.scheme.version:
                g.add((main_concept_scheme, OWL.versionInfo, Literal(obj.scheme.version)))
            if obj.scheme.publisher:
                g.add((main_concept_scheme, DC.publisher, Literal(obj.scheme.publisher)))
            if obj.scheme.relation:
                g.add((main_concept_scheme, DC.relation, URIRef(obj.scheme.relation)))
            if obj.scheme.owner:
                g.add((main_concept_scheme, DCT.rightsHolder, Literal(obj.scheme.owner)))
            g.add((main_concept_scheme, DCT.created, Literal(obj.scheme.date_created, datatype=XSD.dateTime)))
            g.add((main_concept_scheme, DCT.modified, Literal(obj.scheme.date_modified, datatype=XSD.dateTime)))
            if obj.scheme.date_issued:
                g.add((main_concept_scheme, DCT.issued, Literal(obj.scheme.date_issued, datatype=XSD.dateTime)))
            else:
                pass
        else:
            main_concept_scheme = URIRef(VOCABS)
            g.add((main_concept_scheme, RDF.type, SKOS.ConceptScheme))
        # Concept properties
        # TODO user entered URI
        # if obj.legacy_id:
        #     concept = URIRef(obj.legacy_id)
        # else:
        #     concept = URIRef(main_concept_scheme + "#concept" + str(obj.id))
        concept = URIRef(obj.create_uri())
        g.add((concept, RDF.type, SKOS.Concept))
        g.add((concept, SKOS.prefLabel, Literal(obj.pref_label, lang=obj.pref_label_lang)))
        g.add((concept, SKOS.notation, Literal(obj.notation)))
        # each concept must have skos:inScheme main_concept_scheme
        g.add((concept, SKOS.inScheme, main_concept_scheme))
        if obj.collection.all():
            for x in obj.collection.all():
                collection = URIRef(main_concept_scheme + "#collection" + str(x.id))
                g.add((collection, RDF.type, SKOS.Collection))
                g.add((collection, DCT.created, Literal(x.date_created, datatype=XSD.dateTime)))
                g.add((collection, DCT.modified, Literal(x.date_modified, datatype=XSD.dateTime)))
                if x.name:
                    g.add((collection, SKOS.prefLabel, Literal(x.name, lang=x.label_lang)))
                # Collection labels
                if x.has_labels.all():
                    for label in x.has_labels.all():
                        if label.label_type == 'prefLabel':
                            g.add((collection, SKOS.prefLabel, Literal(label.name, lang=label.language)))
                        elif label.label_type == 'altLabel':
                            g.add((collection, SKOS.altLabel, Literal(label.name, lang=label.language)))
                        elif label.label_type == 'hiddenLabel':
                            g.add((collection, SKOS.hiddenLabel, Literal(label.name, lang=label.language)))
                        else:
                            g.add((collection, SKOS.altLabel, Literal(label.name, lang=label.language)))
                # Collection notes
                if x.has_notes.all():
                    for note in x.has_notes.all():
                        if note.note_type == 'note':
                            g.add((collection, SKOS.note, Literal(note.name, lang=note.language)))
                        elif note.note_type == 'scopeNote':
                            g.add((collection, SKOS.scopeNote, Literal(note.name, lang=note.language)))
                        elif note.note_type == 'changeNote':
                            g.add((collection, SKOS.changeNote, Literal(note.name, lang=note.language)))
                        elif note.note_type == 'editorialNote':
                            g.add((collection, SKOS.editorialNote, Literal(note.name, lang=note.language)))
                        elif note.note_type == 'historyNote':
                            g.add((collection, SKOS.historyNote, Literal(note.name, lang=note.language)))
                        elif note.note_type == 'definition':
                            g.add((collection, SKOS.definition, Literal(note.name, lang=note.language)))
                        elif note.note_type == 'example':
                            g.add((collection, SKOS.example, Literal(note.name, lang=note.language)))
                        else:
                            g.add((collection, SKOS.note, Literal(note.name, lang=note.language)))
                # Collection sources
                if x.has_sources.all():
                    for source in x.has_sources.all():
                        g.add((collection, DC.source, Literal(source.name, lang=source.language)))
                if x.creator:
                    for i in x.creator.split(';'):
                        g.add((collection, DC.creator, Literal(i.strip())))
                if x.contributor:
                    for i in x.contributor.split(';'):
                        g.add((collection, DC.contributor, Literal(i.strip())))
                if x.has_members.all():
                    for y in x.has_members.all():
                        if y.legacy_id:
                            g.add((collection, SKOS.member, URIRef(y.legacy_id)))
                        else:
                            g.add((collection, SKOS.member, URIRef(main_concept_scheme + "#concept" + str(y.id))))
        # Concept properties
        if obj.has_labels.all():
            for label in obj.has_labels.all():
                if label.label_type == 'prefLabel':
                    g.add((concept, SKOS.prefLabel, Literal(label.name, lang=label.language)))
                elif label.label_type == 'altLabel':
                    g.add((concept, SKOS.altLabel, Literal(label.name, lang=label.language)))
                elif label.label_type == 'hiddenLabel':
                    g.add((concept, SKOS.hiddenLabel, Literal(label.name, lang=label.language)))
                # if label.label_type is not set then make it altLabel
                else:
                    g.add((concept, SKOS.altLabel, Literal(label.name, lang=label.language)))
        if obj.has_notes.all():
            for note in obj.has_notes.all():
                if note.note_type == 'note':
                    g.add((concept, SKOS.note, Literal(note.name, lang=note.language)))
                elif note.note_type == 'scopeNote':
                    g.add((concept, SKOS.scopeNote, Literal(note.name, lang=note.language)))
                elif note.note_type == 'changeNote':
                    g.add((concept, SKOS.changeNote, Literal(note.name, lang=note.language)))
                elif note.note_type == 'editorialNote':
                    g.add((concept, SKOS.editorialNote, Literal(note.name, lang=note.language)))
                elif note.note_type == 'historyNote':
                    g.add((concept, SKOS.historyNote, Literal(note.name, lang=note.language)))
                elif note.note_type == 'definition':
                    g.add((concept, SKOS.definition, Literal(note.name, lang=note.language)))
                elif note.note_type == 'example':
                    g.add((concept, SKOS.example, Literal(note.name, lang=note.language)))
                else:
                    g.add((concept, SKOS.note, Literal(note.name, lang=note.language)))
        if obj.has_sources.all():
            for source in obj.has_sources.all():
                g.add((concept, DC.source, Literal(source.name, lang=source.language)))
        # top concepts
        if not obj.broader_concept:
            g.add((main_concept_scheme, SKOS.hasTopConcept, URIRef(concept)))
            g.add((concept, SKOS.topConceptOf, main_concept_scheme))
        # modelling broader/narrower relationships
        if obj.broader_concept:
            if obj.broader_concept.legacy_id:
                g.add((concept, SKOS.broader, URIRef(obj.broader_concept.legacy_id)))
            else:
                g.add((concept, SKOS.broader, URIRef(main_concept_scheme + "#concept" + str(obj.broader_concept.id))))
        if obj.narrower_concepts.all():
            for x in obj.narrower_concepts.all():
                if x.legacy_id:
                    g.add((concept, SKOS.narrower, URIRef(x.legacy_id)))
                else:
                    g.add((concept, SKOS.narrower, URIRef(main_concept_scheme + "#concept" + str(x.id))))
        # modelling external matches
        # skos:related
        if obj.related:
            for x in obj.related_as_list():
                g.add((concept, SKOS.related, URIRef(x)))
        # skos:broadMatch
        if obj.broad_match:
            for x in obj.broad_match_as_list():
                g.add((concept, SKOS.broadMatch, URIRef(x)))
        # skos:narrowMatch
        if obj.narrow_match:
            for x in obj.narrow_match_as_list():
                g.add((concept, SKOS.narrowMatch, URIRef(x)))
        # skos:exactMatch
        if obj.exact_match:
            for x in obj.exact_match_as_list():
                g.add((concept, SKOS.exactMatch, URIRef(x)))
        # skos:relatedMatch
        if obj.related_match:
            for x in obj.related_match_as_list():
                g.add((concept, SKOS.relatedMatch, URIRef(x)))
        # skos:closeMatch
        if obj.close_match:
            for x in obj.close_match_as_list():
                g.add((concept, SKOS.closeMatch, URIRef(x)))
        # meta
        if obj.creator:
            for i in obj.creator.split(';'):
                g.add((concept, DC.creator, Literal(i.strip())))
        if obj.contributor:
            for i in obj.contributor.split(';'):
                g.add((concept, DC.contributor, Literal(i.strip())))
        if obj.date_created:
            g.add((concept, DCT.created, Literal(obj.date_created, datatype=XSD.dateTime)))
        if obj.date_modified:
            g.add((concept, DCT.modified, Literal(obj.date_modified, datatype=XSD.dateTime)))
    return g



from rest_framework import viewsets
from rest_framework import pagination
from rest_framework_guardian import filters

from django_filters.rest_framework import DjangoFilterBackend
from .models import *
from .serializers import *
from rest_framework.settings import api_settings
from rest_framework.permissions import DjangoObjectPermissions


class LargeResultsSetPagination(pagination.PageNumberPagination):
    page_size = 25
    page_size_query_param = 'page_size'
    max_page_size = 10000


class SkosConceptSchemeViewSet(viewsets.ModelViewSet):
    queryset = SkosConceptScheme.objects.all()
    serializer_class = SkosConceptSchemeSerializer
    permission_classes = (DjangoObjectPermissions, )
    filter_backends = (filters.ObjectPermissionsFilter, )
    pagination_class = LargeResultsSetPagination


class SkosCollectionViewSet(viewsets.ModelViewSet):
    queryset = SkosCollection.objects.all()
    serializer_class = SkosCollectionSerializer
    permission_classes = (DjangoObjectPermissions, )
    filter_backends = (filters.ObjectPermissionsFilter, )
    pagination_class = LargeResultsSetPagination


class SkosConceptViewSet(viewsets.ModelViewSet):
    queryset = SkosConcept.objects.all()
    serializer_class = SkosConceptSerializer
    filter_backends = (DjangoFilterBackend, filters.ObjectPermissionsFilter,)
    pagination_class = LargeResultsSetPagination
    renderer_classes = tuple(api_settings.DEFAULT_RENDERER_CLASSES)
    permission_classes = (DjangoObjectPermissions, )
